@@ -25,7 +25,6 @@
 #include "midgard_datatypes.h"
 #include "midgard_type.h"
 #include "schema.h"
-#include "midgard_object_class.h"
 #include "midgard_core_config.h"
 #include "midgard_core_object.h"
 #include "midgard_core_object_class.h"
@@ -34,6 +33,8 @@
 #include <libxml/parserInternals.h>
 #include "guid.h"
 #include "midgard_core_xml.h"
+#include "midgard_metadata.h"
+#include "midgard_reflector_object.h"
 
 /* TODO tune header files , no need to include string.h while we need to include midgard.h in almost every case */
 
@@ -132,7 +133,8 @@ static const gchar *mgd_attribute[] = {
 	"index", 
 	"default", 
 	"unique",
-	"multilang", 
+	"multilang",
+	PROP_RW_PRIVATE,
 	NULL 
 };
 
@@ -172,12 +174,6 @@ static const gchar *rcolumns[] = {
 	NULL
 };
 
-/* RESERVED property subnodes' names */
-static const gchar *subnnames[] = {
-	"description",
-	NULL
-};
-
 static gboolean strv_contains(const char **strv, const xmlChar *str) {
         g_assert(strv != NULL);
         g_assert(str != NULL);
@@ -266,13 +262,13 @@ _get_type_attributes(xmlNode * node, MgdSchemaTypeAttr *type_attr, MidgardSchema
 				/* This might be undefined result, so we fallback to NULL default */
 				g_free (type_attr->extends);
 				type_attr->extends = NULL;
-
+			
 			} else {
 
 				MgdSchemaTypeAttr *src = midgard_schema_lookup_type (schema, (gchar *)attrval);
 				midgard_core_schema_type_attr_extend (src, type_attr);
 			}
-	
+
 			xmlFree (table_name);
 			xmlFree (attrval);
 		}
@@ -290,7 +286,7 @@ _get_type_attributes(xmlNode * node, MgdSchemaTypeAttr *type_attr, MidgardSchema
 				g_error("Table definition missed");
 			
 			} else {
-			
+
 				MgdSchemaTypeAttr *src = midgard_schema_lookup_type (schema, (gchar *)attrval);
 				midgard_core_schema_type_attr_extend (src, type_attr);
 			}
@@ -306,18 +302,21 @@ _get_type_attributes(xmlNode * node, MgdSchemaTypeAttr *type_attr, MidgardSchema
 
 			if (g_str_equal (attrval, "false")) {
 
-				type_attr->metadata_class = NULL;
+				type_attr->metadata_class_name = NULL;
+				type_attr->metadata_class_ptr = NULL;
 			
 			} else {
 
-				type_attr->metadata_class = g_strdup((gchar *)attrval);
+				type_attr->metadata_class_name = g_strdup((gchar *)attrval);
+				type_attr->metadata_class_ptr = g_type_class_peek(g_type_from_name((gchar *)attrval));
 			}
 
 			xmlFree(attrval);
 		
 		} else {
 
-			type_attr->metadata_class = g_strdup ("midgard_metadata");
+			type_attr->metadata_class_name = g_strdup ("MidgardMetadata");
+			type_attr->metadata_class_ptr = g_type_class_peek (MIDGARD_TYPE_METADATA);	
 		}
 	}
 }
@@ -500,6 +499,7 @@ midgard_core_schema_get_unique_name(xmlNode * node, MgdSchemaTypeAttr *type_attr
 					gchar *propname = (gchar *)xmlGetProp(node, (const xmlChar *)"name");
 					type_attr->unique_name = g_strdup(propname);
 					g_free(propname);
+					prop_attr->is_unique = TRUE;
 				}
 
 				if (attrval != NULL)
@@ -516,145 +516,183 @@ static void
 _get_property_attributes(xmlNode * node, 
 		MgdSchemaTypeAttr *type_attr, MgdSchemaPropertyAttr *prop_attr)
 {
+	g_return_if_fail (node != NULL);
+	g_return_if_fail (type_attr != NULL);
+	g_return_if_fail (prop_attr != NULL);
+
 	xmlAttr *attr;
-	xmlChar *attrval;
+	xmlChar *attrval = NULL;
+	
+	/* Validate attributes. Throw warning if there's invalid one. */
+	attr = node->properties;
+	attr = attr->next;
 
-	if (node != NULL){
-
-		attr = node->properties;
-		attr = attr->next;
-
-		while (attr != NULL){
-
-			attrval = xmlNodeListGetString (node->doc, attr->children, 1);
-		
-			if(!strv_contains(mgd_attribute, attr->name)){
-				g_warning("Wrong attribute '%s' in '%s' on line %ld",
-						attr->name, parsed_schema, xmlGetLineNo(node));
-			}
-			
-			if(g_str_equal(attr->name, "dbtype"))
-				prop_attr->dbtype = g_strdup((gchar *)attrval);
-			
-			if(g_str_equal(attr->name, "field")){
-				/* Check if column name is reserved one */
-				check_metadata_column(attrval);
-				if (strv_contains(rcolumns, attrval)) {
-					g_critical("'%s' is reserved column name",
-							attrval);
-				}							
-				if (prop_attr->field)
-					g_strdup (prop_attr->field);	
-				prop_attr->field = g_strdup((gchar *)attrval);
-			}
-		
-			if(g_str_equal(attr->name, "index")) {
-				
-				if(g_str_equal(attrval, "yes"))
-					prop_attr->dbindex = TRUE;
-			}
-
-			if(g_str_equal(attr->name, "table")) {
-				/* Disable warning, we can not use debug level */
-				/* __warn_msg (node, "'table' attribute not supported"); */
-			}
-
-			if(g_str_equal(attr->name, "upfield")){
-				check_metadata_column(attrval);
-				/* Check if column name is reserved one */
-				if (strv_contains(rcolumns, attrval)) {
-					g_critical("'%s' is reserved column name",
-							attrval);
-				}				
-				if(!type_attr->property_up){
-					xmlChar *tmpattr = 
-						xmlGetProp (node, (const xmlChar *)"name");
-					type_attr->property_up = g_strdup((gchar *)tmpattr);
-					type_attr->upfield = g_strdup((gchar *)attrval);
-					if(!midgard_core_object_prop_up_is_valid(prop_attr->gtype)) {
-						__warn_msg(node, "Invalid type for up property.");
-						g_error("Wrong schema attribute");
-					}
-					xmlFree(tmpattr);
-				} else {
-					__warn_msg(node, "Upfield redefined!");
-				}
-
-				prop_attr->upfield = g_strdup((gchar *)attrval);			
-			}
-
-			if(g_str_equal(attr->name, "parentfield")){
-				check_metadata_column(attrval);
-				/* Check if column name is reserved one */
-				if (strv_contains(rcolumns, attrval)) {
-					g_critical("'%s' is reserved column name",
-							attrval);
-				}
-				if(!type_attr->property_parent){
-					xmlChar *tmpattr = xmlGetProp (node, (const xmlChar *)"name");
-					type_attr->property_parent = g_strdup((gchar *)tmpattr);
-					type_attr->parentfield = g_strdup((gchar *)attrval);
-					if(!midgard_core_object_prop_parent_is_valid(prop_attr->gtype)) {
-						__warn_msg(node, "Invalid type for parent property.");
-						g_error("Wrong schema attribute");
-					}
-					xmlFree(tmpattr);
-				} else {                         
-					__warn_msg(node, "Parentfield redefined!");
-				}
-				prop_attr->parentfield = g_strdup((gchar *)attrval);
-			}			
-
-			if (g_str_equal(attr->name, "multilang")) {
-				prop_attr->legacy_multilang = TRUE;
-				type_attr->legacy_multilang = TRUE;
-				/* Disable warning, we can not use debug level */
-				/* __warn_msg (node, "'multilang' attribute not supported"); */
-			}
-
-			if(g_str_equal(attr->name, "primaryfield")) {
-				check_metadata_column(attrval);
-				/* Check if column name is reserved one */
-				if (strv_contains(rcolumns, attrval)) {
-					g_critical("'%s' is reserved column name",
-							attrval);
-				}
-				prop_attr->primaryfield = g_strdup((gchar *)attrval);
-				prop_attr->is_primary = TRUE;
-				xmlChar *tmpattr = xmlGetProp (node, (const xmlChar *)"name");
-				type_attr->primary = g_strdup((gchar *)tmpattr);
-				type_attr->primaryfield = g_strdup((gchar *)attrval);
-				xmlFree(tmpattr);
-			}
-
-			if(g_str_equal(attr->name, "reverse")) {
-				if(g_str_equal(attrval, "yes"))
-					prop_attr->is_reversed = TRUE;	
-			}
-			
-			if(g_str_equal(attr->name, "link")) {
-
-				if(!midgard_core_object_prop_link_is_valid(prop_attr->gtype)) {
-					__warn_msg(node, "Invalid type for link property.");
-					g_error("Wrong schema attribute");
-				}
-
-				gchar **link = g_strsplit((gchar *)attrval, ":", -1);
-				prop_attr->link = g_strdup((gchar *)link[0]);
-				prop_attr->is_link = TRUE;
-				if(link[1])
-					prop_attr->link_target = 
-						g_strdup((gchar *)link[1]);
-				else 
-					prop_attr->link_target =
-						g_strdup("guid");
-
-				g_strfreev(link);
-			}
-
-			xmlFree(attrval);
-			attr = attr->next;								
+	while (attr != NULL) {	
+		if (!strv_contains(mgd_attribute, attr->name)) {
+			g_warning ("Wrong attribute '%s' in '%s' on line %ld",
+					attr->name, parsed_schema, xmlGetLineNo(node));
 		}
+		attr = attr->next;
+	}
+	
+	/* dbtype */	
+	attrval = xmlGetProp (node, (const xmlChar *)"dbtype");
+	if (attrval) {
+		prop_attr->dbtype = g_strdup((gchar *)attrval);
+		xmlFree (attrval);
+	}
+
+	/* field */
+	attrval = xmlGetProp (node, (const xmlChar *)PROP_RW_FIELD);
+	if (attrval) {
+		/* Check if column name is reserved one */
+		check_metadata_column (attrval);
+		if (strv_contains (rcolumns, attrval)) {
+			__warn_msg(node, "Invalid (reserved) column name defined");
+			g_error ("Invalid column name");
+		} 
+		prop_attr->field = g_ascii_strdown ((gchar *)attrval, -1);
+		xmlFree (attrval);
+	} else {
+		prop_attr->field = g_ascii_strdown (prop_attr->name, -1);
+	}
+		
+	/* index */
+	attrval = xmlGetProp (node, (const xmlChar *)PROP_RW_INDEX);
+	if (attrval) {
+		if (g_str_equal(attrval, "yes") || g_str_equal (attrval, "true"))
+			prop_attr->dbindex = TRUE;
+		xmlFree (attrval);
+	}
+			
+	/* table, backward compatibility */
+	attrval = xmlGetProp (node, (const xmlChar *)TYPE_RW_TABLE);
+	if (attrval) {
+		/* Disable warning, we can not use debug level */
+		/* __warn_msg (node, "'table' attribute not supported"); */
+		xmlFree (attrval);
+	}
+
+	/* upfield */
+	attrval = xmlGetProp (node, (const xmlChar *)PROP_RW_UPFIELD);
+	if (attrval) {
+		check_metadata_column (attrval);
+		/* Check if column name is reserved one */
+		if (strv_contains (rcolumns, attrval)) {
+			g_critical("'%s' is reserved column name", attrval);
+		}				
+
+		if(!type_attr->property_up) {
+			xmlChar *tmpattr = xmlGetProp (node, (const xmlChar *)"name");
+			type_attr->property_up = g_strdup((gchar *)tmpattr);
+			type_attr->upfield = g_strdup((gchar *)attrval);
+			if (!midgard_core_object_prop_up_is_valid(prop_attr->gtype)) {
+				__warn_msg(node, "Invalid type for up property.");
+				g_error("Wrong schema attribute");
+			}
+			xmlFree(tmpattr);
+
+		} else {
+			__warn_msg(node, "Upfield redefined!");
+		}
+
+		prop_attr->upfield = g_strdup((gchar *)attrval);		
+		xmlFree (attrval);
+	}
+
+	/* parentfield */
+	attrval = xmlGetProp (node, (const xmlChar *)PROP_RW_PARENTFIELD);
+	if (attrval) {
+		check_metadata_column (attrval);
+		/* Check if column name is reserved one */
+		if (strv_contains (rcolumns, attrval)) {
+			g_critical ("'%s' is reserved column name", attrval);
+		}
+
+		if (!type_attr->property_parent) {
+			xmlChar *tmpattr = xmlGetProp (node, (const xmlChar *)"name");
+			type_attr->property_parent = g_strdup ((gchar *)tmpattr);
+			type_attr->parentfield = g_strdup ((gchar *)attrval);
+			if (!midgard_core_object_prop_parent_is_valid (prop_attr->gtype)) {
+				__warn_msg (node, "Invalid type for parent property.");
+				g_error ("Wrong schema attribute");
+			}
+			xmlFree(tmpattr);
+		} else {                         
+			__warn_msg(node, "Parentfield redefined!");
+		}
+
+		if (type_attr->upfield && (g_str_equal (type_attr->upfield, type_attr->parentfield))) {
+			__warn_msg (node, "Parentfield and upfield is the same property");
+			g_error ("Wrong schema attribute");
+		}
+
+		prop_attr->parentfield = g_strdup ((gchar *)attrval);
+		xmlFree (attrval);
+	}
+
+	/* multilang, backward compatibility */
+	attrval = xmlGetProp (node, (const xmlChar *)"multilang");
+	if (attrval) {
+		/* Disable warning, we can not use debug level */
+		/* __warn_msg (node, "'multilang' attribute not supported"); */
+		xmlFree (attrval);
+	}
+
+	/* primaryfield */
+	attrval = xmlGetProp (node, (const xmlChar *)PROP_RW_PRIMARY);
+	if (attrval) {
+		check_metadata_column (attrval);
+		
+		/* Check if column name is reserved one */
+		if (strv_contains (rcolumns, attrval)) 
+			g_critical ("'%s' is reserved column name",attrval);
+
+		prop_attr->primaryfield = g_strdup ((gchar *)attrval);
+		prop_attr->is_primary = TRUE;
+		xmlChar *tmpattr = xmlGetProp (node, (const xmlChar *)"name");
+		type_attr->primary = g_strdup ((gchar *)tmpattr);
+		type_attr->primaryfield = g_strdup ((gchar *)attrval);
+		xmlFree (tmpattr);
+		xmlFree (attrval);
+	}
+
+	/* reverse */
+	attrval = xmlGetProp (node, (const xmlChar *)PROP_RW_REVERSE);
+	if (attrval ) {
+		if (g_str_equal (attrval, "yes") || g_str_equal (attrval, "true"))
+			prop_attr->is_reversed = TRUE;	
+			
+		xmlFree (attrval);
+	}
+		
+	/* link */	
+	attrval = xmlGetProp (node, (const xmlChar *)PROP_RW_LINK);
+	if (attrval) {
+		if(!midgard_core_object_prop_link_is_valid(prop_attr->gtype)) {
+			__warn_msg(node, "Invalid type for link property.");
+			g_error("Wrong schema attribute");
+		}
+
+		gchar **link = g_strsplit((gchar *)attrval, ":", -1);
+		prop_attr->link = g_strdup((gchar *)link[0]);
+		prop_attr->is_link = TRUE;
+		
+		if(link[1])
+			prop_attr->link_target = g_strdup((gchar *)link[1]);
+		else 
+			prop_attr->link_target = g_strdup("guid");
+		
+		g_strfreev(link);
+		xmlFree (attrval);
+	}
+
+	/* private */
+	attrval = xmlGetProp (node, (const xmlChar *)PROP_RW_PRIVATE);
+	if (attrval) {
+		if (g_str_equal (attrval, "yes") || g_str_equal (attrval, "true"))
+			prop_attr->is_private = TRUE;
+		xmlFree (attrval);
 	}
 }
 
@@ -758,13 +796,11 @@ __get_properties (xmlNode *curn, MgdSchemaTypeAttr *type_attr, MidgardSchema *sc
 			}
 
 			prop_attr = midgard_core_schema_type_property_attr_new();
+			prop_attr->name = g_strdup ((gchar *)nv);
 			midgard_core_schema_get_property_type (node, type_attr, prop_attr);
 			midgard_core_schema_get_default_value (node, type_attr, prop_attr);
 			midgard_core_schema_get_unique_name (node, type_attr, prop_attr);
 			_get_user_fields (node, prop_attr);
-			prop_attr->name = g_strdup ((gchar *)nv);
-			/* "Initialize" field name in case it's not configured in schema */
-			prop_attr->field = g_strdup (prop_attr->name);
 			_get_property_attributes (node, type_attr, prop_attr);
 
 			if(prop_attr->is_primary 
@@ -887,7 +923,7 @@ void __get_pdata_foreach(gpointer key, gpointer value, gpointer user_data)
 		midgard_core_schema_type_property_set_tablefield(prop_attr, table, parentfield);
 
 	if(prop_attr->field == NULL)
-		prop_attr->field = g_strdup(pname);
+		prop_attr->field = g_ascii_strdown (pname, -1);
 
 	/* Force G_TYPE_UINT type for primaryfield */
 	if((prop_attr->is_primary) && (prop_attr->gtype != G_TYPE_UINT))
@@ -929,7 +965,7 @@ void _copy_schemas(gpointer key, gpointer value, gpointer userdata)
 {
 	gchar *typename = (gchar *)key;
 	MgdSchemaTypeAttr *type = (MgdSchemaTypeAttr *)value;
-	MgdSchema *ts = (MgdSchema *) userdata;
+	MidgardSchema *ts = (MidgardSchema *) userdata;
 
 	if(!g_hash_table_lookup(ts->types, typename)) {
 		
@@ -1142,7 +1178,7 @@ static void parse_schema(gpointer key, gpointer value, gpointer user_data) {
  * @path: full path to a xml file with common classes
  *
  * Reads xml file which defines very basic and common classes.
- * By default it's `/usr/local/share/midgard/MgdObjects.xml` file.
+ * By default it's `/usr/local/share/midgard/MidgardObjects.xml` file.
  */
 void midgard_schema_init(MidgardSchema *self, const gchar *path)
 {	
@@ -1160,13 +1196,13 @@ void midgard_schema_init(MidgardSchema *self, const gchar *path)
 
 	if (g_file_test(lpath, G_FILE_TEST_IS_DIR)) {
 
-		g_warning("Common MgdObjects.xml path is a directory");
+		g_warning("Common MidgardObjects.xml path is a directory");
 		return;
 	}
 	
 	if (!g_file_test(lpath, G_FILE_TEST_EXISTS)) {
 
-		g_warning("Common MgdObjects.xml file (%s) doesn't exist.", lpath);
+		g_warning("Common MidgardObjects.xml file (%s) doesn't exist.", lpath);
 		return;
 	}
 
@@ -1293,7 +1329,7 @@ static void __midgard_schema_validate()
 
 		g_free(pspecs);
 
-		parentname = klass->dbpriv->storage_data->parent;
+		parentname = MIDGARD_DBOBJECT_CLASS (klass)->dbpriv->storage_data->parent;
 
 		if(parentname != NULL) {
 			
@@ -1305,9 +1341,9 @@ static void __midgard_schema_validate()
 				g_error("Parent %s for %s class is not registered in GType system",
 						parentname, typename);
 			}
-		
+	
 			/* validate parent property if exists */
-			const gchar *prop = midgard_object_class_get_property_parent(klass);	
+			const gchar *prop = midgard_reflector_object_get_property_parent (typename);	
 			MidgardReflectionProperty *smrp = NULL;
 	
 			if(prop) {
@@ -1322,7 +1358,7 @@ static void __midgard_schema_validate()
 			}
 			
 			/* validate up property link */
-			prop = midgard_object_class_get_property_up(klass);
+			prop = midgard_reflector_object_get_property_up (typename);
 			
 			if(prop) {
 				
@@ -1374,7 +1410,7 @@ static void __extend_type_foreach(gpointer key, gpointer val, gpointer userdata)
 	type_attr->table = g_strdup(parent_type_attr->table);
 	type_attr->tables = g_strdup(parent_type_attr->tables);
 
-	GSList *slist = NULL;
+        GSList *slist = NULL;
 
 	for (slist = parent_type_attr->_properties_list; slist != NULL; slist = slist->next) {
 		type_attr->_properties_list = g_slist_prepend (type_attr->_properties_list, slist->data);
@@ -1420,7 +1456,7 @@ static void __copy_type_foreach(gpointer key, gpointer val, gpointer userdata)
 	if (!parent_type_attr)
 		g_error("Type information for %s (%s's parent) not found", 
 				copied, type_attr->name);
-	GSList *slist = NULL;
+        GSList *slist = NULL;
 
 	for (slist = parent_type_attr->_properties_list; slist != NULL; slist = slist->next) {
 		type_attr->_properties_list = g_slist_prepend (type_attr->_properties_list, slist->data);
@@ -1631,16 +1667,10 @@ static void
 _midgard_schema_class_init (gpointer g_class, gpointer g_class_data)
 {
 	GObjectClass *gobject_class = G_OBJECT_CLASS (g_class);
-	MidgardSchemaClass *klass = MIDGARD_SCHEMA_CLASS (g_class);
 	
 	gobject_class->set_property = NULL;
 	gobject_class->get_property = NULL;
-	gobject_class->finalize = _midgard_schema_finalize;
-	
-	klass->init = midgard_schema_init;
-	klass->read_dir = midgard_schema_read_dir;
-	klass->read_file = midgard_schema_read_file;
-	klass->type_exists = midgard_schema_type_exists;
+	gobject_class->finalize = _midgard_schema_finalize;	
 }
 
 /* Register MidgardSchema type */
@@ -1660,9 +1690,7 @@ midgard_schema_get_type (void)
 			0,              /* n_preallocs */
 			_schema_instance_init    /* instance_init */
 		};
-		type = g_type_register_static (G_TYPE_OBJECT,
-				"midgard_schema",
-				&info, 0);
+		type = g_type_register_static (G_TYPE_OBJECT, "MidgardSchema", &info, 0);
 	}
 	return type;
 }
