@@ -41,7 +41,7 @@ static void __get_filepath(MidgardBlob *self)
 	gchar *fname = NULL;
 	gchar *location = NULL;
 	gchar *up_a, *up_b;
-	MgdObject *attachment = self->priv->attachment;
+	MidgardObject *attachment = self->priv->attachment;
 
 	GParamSpec *pspec =
 		g_object_class_find_property(
@@ -132,14 +132,14 @@ static void __get_channel(MidgardBlob *self, const gchar *mode)
 
 /**
  * midgard_blob_new: 
- * @attachment: #MgdObject of MIDGARD_TYPE_ATTACHMENT type.
+ * @attachment: #MidgardObject of MIDGARD_TYPE_ATTACHMENT type.
  * @encoding: file encoding
  * 
  * Default encoding is UTF-8. Set NULL @encoding if such is required. 
  *
  * Instatiate new Midgard Blob object for the given midgard_attachment object. 
  * This is almost the same constructor as g_object_new, but unlike that one,
- * midgard_blob_new requires MgdObject (midgard_attachment) object's pointer.
+ * midgard_blob_new requires MidgardObject (midgard_attachment) object's pointer.
  *
  * This constructor defines new relative path for attachment, if midgard_attachment 
  * is associated with midgard_blob and its location is empty. 
@@ -147,11 +147,11 @@ static void __get_channel(MidgardBlob *self, const gchar *mode)
  * 
  * Returns: newly instatiated #MidgardBlob object or %NULL on failure
  */
-MidgardBlob *midgard_blob_new(MgdObject *attachment, const gchar *encoding)
+MidgardBlob *midgard_blob_new (MidgardObject *attachment, const gchar *encoding)
 {
 	g_assert(attachment != NULL);
 	
-	MidgardConnection *mgd = attachment->dbpriv->mgd;
+	MidgardConnection *mgd = MGD_OBJECT_CNC (attachment);
 
 	if(mgd == NULL) {
 		g_critical("MidgardConnection not found for given attachment");
@@ -160,12 +160,12 @@ MidgardBlob *midgard_blob_new(MgdObject *attachment, const gchar *encoding)
 
 	MIDGARD_ERRNO_SET(mgd, MGD_ERR_OK);
 
-	const gchar *blobdir = attachment->dbpriv->mgd->priv->config->blobdir;
+	const gchar *blobdir = MIDGARD_DBOBJECT (attachment)->dbpriv->mgd->priv->config->blobdir;
 	if(!g_file_test(blobdir,G_FILE_TEST_EXISTS)) {
 		midgard_set_error(mgd,
 				MGD_GENERIC_ERROR,
 				MGD_ERR_INTERNAL,
-				" Blobs directory doesn't exist. %s", attachment->dbpriv->mgd->priv->config->blobdir);
+				" Blobs directory doesn't exist. %s", blobdir);
 		return NULL;
 	}
 
@@ -176,7 +176,7 @@ MidgardBlob *midgard_blob_new(MgdObject *attachment, const gchar *encoding)
 
 	MidgardBlob *self = g_object_new(MIDGARD_TYPE_BLOB, NULL);
 	self->priv->attachment = attachment;
-	self->priv->mgd = attachment->dbpriv->mgd;
+	self->priv->mgd = mgd;
 	self->priv->blobdir = g_strdup(blobdir);
 	self->priv->channel = NULL;
 	self->priv->encoding = NULL;
@@ -627,9 +627,7 @@ GType midgard_blob_get_type(void)
 			0,              /* n_preallocs */
 			(GInstanceInitFunc) _midgard_blob_instance_init /* instance_init */
 		};
-		type = g_type_register_static (G_TYPE_OBJECT,
-				"midgard_blob",
-				&info, 0);
+		type = g_type_register_static (G_TYPE_OBJECT, "MidgardBlob", &info, 0);
 	}
 	return type;
 }
