@@ -38,7 +38,6 @@ G_BEGIN_DECLS
 	(G_TYPE_INSTANCE_GET_CLASS ((obj), MIDGARD_TYPE_BLOB, MidgardBlobClass))
 
 typedef struct MidgardBlob MidgardBlob;
-typedef struct MidgardBlob midgard_blob;
 typedef struct MidgardBlobClass MidgardBlobClass;
 typedef struct _MidgardBlobPrivate MidgardBlobPrivate;
 
@@ -60,7 +59,7 @@ struct MidgardBlob {
 };
 
 GType 		midgard_blob_get_type		(void);
-MidgardBlob 	*midgard_blob_new		(MgdObject *attachment, const gchar *encoding);
+MidgardBlob 	*midgard_blob_new		(MidgardObject *attachment, const gchar *encoding);
 gchar 		*midgard_blob_read_content	(MidgardBlob *self, gsize *bytes_read);
 gboolean 	midgard_blob_write_content	(MidgardBlob *self, const gchar *content);
 GIOChannel 	*midgard_blob_get_handler	(MidgardBlob *self, const gchar *mode);
