@@ -33,17 +33,19 @@
  * @short_descritpion: MidgardConnection description
  */ 
 
+#ifdef HAVE_LIBGDA_4
+#include <sql-parser/gda-sql-parser.h>
+#endif
+
 #define MGD_MYSQL_HOST "127.0.0.1"
 #define MGD_MYSQL_DATABASE "midgard"
 #define MGD_MYSQL_USERNAME "midgard"
 #define MGD_MYSQL_PASSWORD "midgard"
 
-gboolean __midgard_connection_open(
-		MidgardConnection *mgd,
-		GHashTable **hashtable, gboolean init_schema);
+gboolean 
+__midgard_connection_open (MidgardConnection *mgd, GHashTable **hashtable, gboolean init_schema);
 
-static 
-MidgardConnectionPrivate *
+static MidgardConnectionPrivate *
 midgard_connection_private_new (void)
 {
 	MidgardConnectionPrivate *cnc_private = 
@@ -52,10 +54,8 @@ midgard_connection_private_new (void)
 	cnc_private->pattern = NULL;	
 	cnc_private->config = NULL;
 	cnc_private->connected = FALSE;
-	cnc_private->free_config = FALSE;
 	cnc_private->loghandler = 0;
-	cnc_private->loglevel = 0;
-	cnc_private->person = NULL;
+	cnc_private->loglevel = 0;	
 	cnc_private->user = NULL;
 	cnc_private->inherited = FALSE;
 #ifdef HAVE_LIBGDA_4
@@ -64,7 +64,6 @@ midgard_connection_private_new (void)
 	cnc_private->client = NULL;
 #endif
 	cnc_private->connection = NULL;
-	cnc_private->cache = NULL;
 	cnc_private->configname = NULL;
 	cnc_private->cnc_str = NULL;
 
@@ -78,53 +77,55 @@ midgard_connection_private_new (void)
 	cnc_private->timer = g_timer_new();
 	g_timer_start(cnc_private->timer);	
 
+	cnc_private->enable_replication = FALSE;
+	cnc_private->enable_quota = FALSE;
+	cnc_private->enable_debug = FALSE;
+	cnc_private->enable_dbus = FALSE;
+
 	return cnc_private;
 }
 
-static void 
-__midgard_connection_struct_free (MidgardConnection *self, gboolean object)
-{
-	if (self->err != NULL)
-		g_clear_error (&self->err);
-
-	if (self->errstr != NULL) {
-		g_free (self->errstr);
-		self->errstr = NULL;
-	}
-
-	if (self->priv->authstack) {
-		g_slist_free (self->priv->authstack);
-		self->priv->authstack = NULL;
-	}
-
-	if (self->priv->cache) {
-		g_free (self->priv->cache);
-		self->priv->cache = NULL;
-	}
-
-	g_free ((gchar *)self->priv->cnc_str);
-	self->priv->cnc_str = NULL;
-
-	g_timer_stop (self->priv->timer);
-	/* gdouble bench = g_timer_elapsed (self->priv->timer, NULL); */
-
-	if (self->priv->loghandler) {
-		/* g_message ("MidgardConnection uptime %.04f seconds", bench); */
-		g_log_remove_handler (G_LOG_DOMAIN, self->priv->loghandler);
-	}
-
-	g_timer_destroy (self->priv->timer);
-
-	g_free (self->priv);
-	self->priv = NULL;
-}
-
 static void _midgard_connection_finalize(GObject *object)
 {
 	g_assert(object != NULL);
 	MidgardConnection *self = (MidgardConnection *) object;
 
-	__midgard_connection_struct_free(self, TRUE);
+	if (self->err != NULL)
+		g_clear_error(&self->err);
+
+	if (self->errstr != NULL) {
+		g_free(self->errstr);
+		self->errstr = NULL;
+	}
+
+	if (self->priv->authstack) {
+		g_slist_free(self->priv->authstack);
+		self->priv->authstack = NULL;
+	}
+
+	g_free ((gchar *)self->priv->cnc_str);
+	self->priv->cnc_str = NULL;
+
+	g_timer_stop(self->priv->timer);
+	//gdouble bench = g_timer_elapsed(self->priv->timer, NULL);
+
+	if (self->priv->loghandler) {
+		/* uptime message disable for a while */
+		/* g_message("MidgardConnection uptime %.04f seconds", bench); */
+		g_log_remove_handler(G_LOG_DOMAIN, self->priv->loghandler);
+	}
+
+	g_timer_destroy(self->priv->timer);
+
+	if (!self->priv->inherited) {
+		if (self->priv->config && G_IS_OBJECT (self->priv->config)) {
+			g_object_unref(self->priv->config);
+			self->priv->config = NULL;
+		}
+	}
+
+	g_free(self->priv);
+	self->priv = NULL;
 }
 
 static void _midgard_connection_dispose(GObject *object)
@@ -136,25 +137,14 @@ static void _midgard_connection_dispose(GObject *object)
 	GdaClient *gda_client = NULL;
 #endif
 
-	if (self->priv->person != NULL) {
-		g_object_unref(self->priv->person);
-		self->priv->person = NULL;
-	}
-
 	while (self->priv->user != NULL) {
 		// emptying authstack
-		gboolean res = midgard_user_logout((MidgardUser *)self->priv->user);
+		gboolean res = midgard_user_log_out((MidgardUser *)self->priv->user);
 		g_assert(res == TRUE);
 	}
 
 	/* Free only these data which are not inherited */
 	if (!self->priv->inherited) {
-		if(self->priv->free_config) {
-			g_object_unref(self->priv->config);
-			self->priv->config = NULL;
-			self->priv->free_config = FALSE;
-		}
-
 #ifdef HAVE_LIBGDA_4
 		if(self->priv->parser != NULL)
 			g_object_unref(self->priv->parser);
@@ -182,45 +172,6 @@ static void _midgard_connection_dispose(GObject *object)
 	}
 }
 
-enum {	
-	_MIDGARD_CNC_USER = 1,
-	_MIDGARD_CNC_PERSON,
-	_MIDGARD_CNC_ERRSTR,
-	_MIDGARD_CNC_ERRNUM
-};
-
-static void
-_midgard_connection_get_property (GObject *object, guint property_id,
-		GValue *value, GParamSpec *pspec)
-{
-	MidgardConnection *self = (MidgardConnection *) object;
-	
-	switch (property_id) {
-		
-		case _MIDGARD_CNC_USER:
-			if(self->priv->user != NULL)
-				g_value_set_object(value, MIDGARD_USER(self->priv->user));
-			break;
-
-		case _MIDGARD_CNC_PERSON:
-			if(self->priv->person != NULL)
-				g_value_set_object(value, self->priv->person);
-			break;
-
-		case _MIDGARD_CNC_ERRSTR:
-			g_value_set_string(value, self->errstr);
-			break;
-
-		case _MIDGARD_CNC_ERRNUM:
-			g_value_set_int(value, self->errnum);
-			break;
-
-		default:
-			G_OBJECT_WARN_INVALID_PROPERTY_ID(object,property_id,pspec);
-			break;
-	}
-}
-
 static void _midgard_connection_class_init(
 		gpointer g_class, gpointer g_class_data)
 {
@@ -229,13 +180,6 @@ static void _midgard_connection_class_init(
 
 	gobject_class->finalize = _midgard_connection_finalize;
 	gobject_class->dispose = _midgard_connection_dispose;
-	gobject_class->get_property = _midgard_connection_get_property;
-	klass->open = midgard_connection_open;
-	klass->open_config = midgard_connection_open_config;
-	klass->set_loglevel = midgard_connection_set_loglevel;
-	klass->get_loglevel = midgard_connection_get_loglevel;
-	klass->set_loghandler = midgard_connection_set_loghandler;
-	klass->get_loghandler = midgard_connection_get_loghandler;
 
 	/* signals */
 	/**
@@ -342,9 +286,7 @@ GType midgard_connection_get_type(void)
 			0,              /* n_preallocs */
 			(GInstanceInitFunc) _midgard_connection_instance_init /* instance_init */
 		};
-		type = g_type_register_static (G_TYPE_OBJECT,
-				"midgard_connection",
-				&info, 0);
+		type = g_type_register_static (G_TYPE_OBJECT, "MidgardConnection", &info, 0);
 	}
 	return type;
 }
@@ -499,43 +441,17 @@ gboolean __midgard_connection_open(
 #endif
 	midgard_connection_set_loglevel(mgd, loglevel, NULL);
 
-	if (hashtable == NULL && mgd->priv->cache != NULL) {
-
-		g_critical("Tried to assign NULL holder for already allocated cache");
-		MIDGARD_ERRNO_SET (mgd, MGD_ERR_INTERNAL);
-		return FALSE;
-	}
-
-	/* Initialize hashtable and holder. It's initialized per MidgardConnection instance */
-	if (hashtable != NULL && *hashtable == NULL) {
-	
-		*hashtable = g_hash_table_new_full(
-				g_str_hash, g_direct_equal,
-				(GDestroyNotify)g_free, NULL);
-	}
-
-	/* Look for cache and reuse it if found */
-	gchar *dbathost = g_strconcat(config->database, "at", config->host, NULL);
-
-	if (hashtable && *hashtable)
-		mgd->priv->cache = g_hash_table_lookup(*hashtable, (gchar *)dbathost);
-	
-	if (hashtable && *hashtable && mgd->priv->cache == NULL) {
-
-		mgd->priv->cache = g_new(MidgardConnectionCache, 1);
-		mgd->priv->cache->sg_ids = NULL;
-		g_datalist_init(&mgd->priv->cache->sg_datalist);
-		g_hash_table_insert(*hashtable,  dbathost, (gpointer) mgd->priv->cache);
-
-	} else {
-
-		g_free(dbathost);
-	}
-
 	if(config->priv->dbtype == MIDGARD_DB_TYPE_SQLITE) {
 
-		const gchar *sqlite_dir[] = {"data", NULL};
-		gchar *path = midgard_core_config_build_path(sqlite_dir, NULL, TRUE);
+		gchar *path = NULL;
+		gchar *dbdir = config->dbdir;
+		if (!dbdir || *dbdir == '\0') {
+			const gchar *sqlite_dir[] = {"data", NULL};
+			path = midgard_core_config_build_path(sqlite_dir, NULL, TRUE);
+		} else {
+			path = g_strdup(dbdir);
+		}
+
 		tmpstr = g_strconcat("DB_DIR=", path, ";", "DB_NAME=", dbname, NULL);
 		g_free(path);
 
@@ -614,7 +530,12 @@ gboolean __midgard_connection_open(
 
 	g_free(tmpstr);
 
-#ifndef HAVE_LIBGDA_4
+#ifdef HAVE_LIBGDA_4
+	mgd->priv->parser = gda_connection_create_parser (connection);
+	if (!mgd->priv->parser)
+		mgd->priv->parser = gda_sql_parser_new();
+	g_assert (mgd->priv->parser != NULL);
+#else 
 	mgd->priv->client = client;
 #endif
 
@@ -623,10 +544,10 @@ gboolean __midgard_connection_open(
 
 	if(init_schema) {
 		
-		if(!g_type_from_name("midgard_language")) {
+		if(!g_type_from_name("midgard_quota")) {
 			
 			MidgardSchema *schema = g_object_new(MIDGARD_TYPE_SCHEMA, NULL);
-			gchar *path = g_build_path(G_DIR_SEPARATOR_S, config->sharedir, "MgdObjects.xml", NULL);
+			gchar *path = g_build_path(G_DIR_SEPARATOR_S, config->sharedir, "MidgardObjects.xml", NULL);
 			midgard_schema_init(schema, (const gchar *)path);
 			g_free(path);
 			midgard_schema_read_dir(schema, config->sharedir);
@@ -665,7 +586,7 @@ gboolean __midgard_connection_open(
  * error state.
  *
  * It also initializes #MidgardSchema object (which is encapsulated by implementation )
- * and register all MgdSchema, #MgdObjectClass derived classes defined by user.
+ * and register all MgdSchema, #MidgardObjectClass derived classes defined by user.
  * This happens only when basic Midgard classes are not registered in GType system.
  * This is recommended way to initialize MgdSchema types.
  *  
@@ -706,7 +627,6 @@ midgard_connection_open (MidgardConnection *self, const char *name, GError **err
 		g_clear_error(&rf_error);
 
 	self->priv->config = config;
-	self->priv->free_config = TRUE;
 
 	GHashTable *hash = NULL;
 	gboolean rv = TRUE;
@@ -767,7 +687,6 @@ gboolean midgard_connection_open_from_file(
 		g_clear_error(&rf_error);
 
 	mgd->priv->config = config;
-	mgd->priv->free_config = TRUE;
 
 	GHashTable *hash = NULL;
 	gboolean rv = TRUE;
@@ -823,7 +742,6 @@ extern GHashTable *midgard_connection_open_all(gboolean userdir)
 		}
 
 		cnc->priv->config = config;
-		cnc->priv->free_config = TRUE;
 
 		if(__midgard_connection_open(cnc, &hash, TRUE)) {
 
@@ -866,16 +784,20 @@ gboolean midgard_connection_open_config(
 	g_assert(self != NULL);	
 	g_assert(config != NULL);
 
-	if (self->priv->config && (self->priv->config == config))
+	MidgardConfig *self_config = self->priv->config;
+
+	/* Emulate the same config pointer, as we have copy associated */
+	if (self_config 
+			&& (g_str_equal (self_config->database, config->database)
+				&& g_str_equal (self_config->dbtype, config->dbtype)
+				&& g_str_equal (self_config->host, config->host))) {
 		return TRUE;
-
-	if (self->priv->config && (self->priv->config != config)) {
-
+	} else if (self_config) {
 		MIDGARD_ERRNO_SET_STRING (self, MGD_ERR_INTERNAL, "Midgard connection already associated with configuration");
 		return FALSE;
 	}
 
-	self->priv->config = config;
+	self->priv->config = midgard_config_copy (config);
 
 	GHashTable *hash = NULL;
 	gboolean rv = TRUE;
@@ -941,7 +863,12 @@ gboolean midgard_connection_set_loglevel(
 		self->priv->loglevel = loglevel;
 	else
 		return FALSE;
-	
+
+	MGD_CNC_DEBUG (self) = FALSE;
+
+	if (loglevel > G_LOG_LEVEL_DEBUG)
+		MGD_CNC_DEBUG (self) = TRUE;
+
 	guint loghandler = midgard_connection_get_loghandler(self);
 	if(loghandler)
 		g_log_remove_handler(G_LOG_DOMAIN, loghandler);
@@ -1198,25 +1125,6 @@ midgard_connection_is_connected (MidgardConnection *self)
 	return self->priv->connected;
 }
 
-/* !! Undocumented workarounds !! */
-
-MidgardConnection *midgard_connection_struct_new(void) 
-{
-	MidgardConnection *self = 
-		g_new0(MidgardConnection, 1);
-	self->errstr = g_strdup("MGD_ERR_OK");
-	self->err = NULL;
-	self->priv = midgard_connection_private_new(); 
-	
-	return self;
-}
-
-void midgard_connection_struct_free(MidgardConnection *self)
-{
-	__midgard_connection_struct_free(self, FALSE);
-}
-
-
 /**
  * midgard_connection_copy:
  * @self: #MidgardConnection instance
@@ -1249,6 +1157,11 @@ MidgardConnection *midgard_connection_copy(MidgardConnection *self)
 
 	new_mgd->priv->authtypes = self->priv->authtypes;
 
+	MGD_CNC_QUOTA (new_mgd) = MGD_CNC_QUOTA (self);
+	MGD_CNC_DBUS (new_mgd) = MGD_CNC_DBUS (self);
+	MGD_CNC_REPLICATION (new_mgd) = MGD_CNC_REPLICATION (self);
+	MGD_CNC_DEBUG (new_mgd) = MGD_CNC_DEBUG (self);
+
 	/* Set pattern pointer. This is important (at least) for signals.
 	 * If we need to disconnect from signal at some point (e.g. indexes creation)
 	 * we should disconnect copy and original one. */
@@ -1263,3 +1176,99 @@ MidgardConnection *midgard_connection_copy(MidgardConnection *self)
 
 	return new_mgd;
 }
+
+/**
+ * midgard_connection_enable_quota:
+ * @self: #MidgardConnection instance
+ * @toggle: quota enable, disable toggle
+ *
+ * Enable or disable quota table usage.
+ * If enabled, every base operation (create, update, delete) will be recorded in quota table, 
+ * limiting particular types usage.
+ *
+ * Since: 10.05
+ */ 
+void
+midgard_connection_enable_quota (MidgardConnection *self, gboolean toggle)
+{
+	g_return_if_fail (self != NULL);
+	self->priv->enable_quota = toggle;
+}
+
+/**
+ * midgard_connection_enable_replication:
+ * @self: #MidgardConnection instance
+ * @toggle: replication enable, disable toggle
+ *
+ * Enable or disable repligard table usage.
+ * If enabled, every base operation (create, update, delete) will be recorded in repligard table.
+ *
+ * Since: 10.05
+ */ 
+void
+midgard_connection_enable_replication (MidgardConnection *self, gboolean toggle)
+{
+	g_return_if_fail (self != NULL);
+	self->priv->enable_replication = toggle;
+}
+
+/**
+ * midgard_connection_enable_dbus:
+ * @self: #MidgardConnection instance
+ * @toggle: dbus enable, disable toggle
+ *
+ * Enable or disable dbus messages send for basic operation
+ *
+ * Since: 10.05
+ */ 
+void
+midgard_connection_enable_dbus (MidgardConnection *self, gboolean toggle)
+{
+	g_return_if_fail (self != NULL);
+	self->priv->enable_dbus = toggle;
+}
+
+/**
+ * midgard_connection_is_enabled_quota:
+ * @self: #MidgardConnection instance
+ *
+ * Returns: %TRUE, if quota is enabled, %FALSE otherwise
+ * 
+ * Since: 10.05
+ */ 
+gboolean
+midgard_connection_is_enabled_quota (MidgardConnection *self)
+{
+	g_return_val_if_fail (self != NULL, FALSE);
+	return self->priv->enable_quota;
+}
+
+/**
+ * midgard_connection_is_enabled_replication:
+ * @self: #MidgardConnection instance
+ *
+ * Returns: %TRUE, if replication is enabled, %FALSE otherwise
+ * 
+ * Since: 10.05
+ */ 
+gboolean
+midgard_connection_is_enabled_replication (MidgardConnection *self)
+{
+	g_return_val_if_fail (self != NULL, FALSE);
+	return self->priv->enable_replication;
+}
+
+/**
+ * midgard_connection_is_enabled_dbus:
+ * @self: #MidgardConnection instance
+ *
+ * Returns: %TRUE, if dbus is enabled, %FALSE otherwise
+ * 
+ * Since: 10.05
+ */ 
+gboolean                
+midgard_connection_is_enabled_dbus (MidgardConnection *self)
+{
+	g_return_val_if_fail (self != NULL, FALSE);
+	return self->priv->enable_dbus;
+}
