@@ -33,10 +33,11 @@
 #include "midgard_blob.h"
 #include "midgard_user.h"
 #include "midgard_object.h"
-#include "midgard_object_class.h"
 #include "midgard_metadata.h"
 #include "guid.h"
 #include "midgard_core_metadata.h"
+#include "midgard_core_object_class.h"
+#include "midgard_schema_object_factory.h"
 
 static const gchar *MIDGARD_OBJECT_HREF = "http://www.midgard-project.org/midgard_object/1.8";
 
@@ -46,7 +47,7 @@ static void _write_nodes(GObject *object, xmlNodePtr node)
 	g_assert(node);
 
 	guint prop_n;
-	MgdObject *mgdobject = (MgdObject *)object;
+	MidgardObject *mgdobject = (MidgardObject *)object;
 	MidgardReflectionProperty *mrp = NULL;
 	MidgardObjectClass *klass = NULL;
 	GParamSpec **pspec = g_object_class_list_properties(
@@ -76,17 +77,17 @@ static void _write_nodes(GObject *object, xmlNodePtr node)
 
 		gint object_action = -1;	
 
-		if(mgdobject->dbpriv->guid) {
+		if(MGD_OBJECT_GUID (mgdobject)) {
 
 			GString *_sql = g_string_new(" ");
 			g_string_append_printf(_sql, "guid = '%s' ",	
-					mgdobject->dbpriv->guid);
+					MGD_OBJECT_GUID (mgdobject));
 			
 			gchar *tmpstr = g_string_free(_sql, FALSE);
 						
 			GValue *avalue =
 				midgard_core_query_get_field_value(
-						mgdobject->dbpriv->mgd,	
+						MGD_OBJECT_CNC (mgdobject),	
 						"object_action",
 						"repligard", 
 						(const gchar*)tmpstr);
@@ -141,13 +142,13 @@ static void _write_nodes(GObject *object, xmlNodePtr node)
 		if(g_str_equal("guid", pspec[i]->name)) {
 			/* Add guid attribute */
 			xmlNewProp(node, BAD_CAST "guid", 
-					BAD_CAST MIDGARD_OBJECT(object)->dbpriv->guid);
+					BAD_CAST MGD_OBJECT_GUID (object));
 			g_value_unset(&pval);
 			continue;
 		}
 				
 		/* Object is not fetched from database. Skip references */
-		if(mgdobject->dbpriv->guid == NULL)
+		if(MGD_OBJECT_GUID (mgdobject) == NULL)
 			goto export_unchecked_property;
 
 		/* If property is a link we need to query guid 
@@ -193,7 +194,7 @@ static void _write_nodes(GObject *object, xmlNodePtr node)
 				
 				if(linktype){
 					mc = midgard_collector_new(
-							mgdobject->dbpriv->mgd,
+							MGD_OBJECT_CNC (mgdobject),
 							linktype,
 							"id",
 							lval);
@@ -233,7 +234,7 @@ static void _write_nodes(GObject *object, xmlNodePtr node)
 		}	
 		
 		export_unchecked_property:
-		switch(pspec[i]->value_type) {
+		switch (G_TYPE_FUNDAMENTAL(pspec[i]->value_type)) {
 			
 			case G_TYPE_STRING:				
 				strprop = g_value_dup_string(&pval);
@@ -314,7 +315,6 @@ static void _write_nodes(GObject *object, xmlNodePtr node)
 						BAD_CAST (xmlChar *)g_value_get_string (&strval));
 					g_value_unset (&strval);
 
-
 				} else {
 					g_warning ("midgard_replicator_serialize: unhandled %s property type (%s)", 
 							pspec[i]->name, g_type_name (pspec[i]->value_type)); 
@@ -403,8 +403,8 @@ gboolean _nodes2object(GObject *object, xmlNode *node, gboolean force)
 	gchar *nodeprop = NULL;
 	xmlChar *decoded;
 	xmlParserCtxtPtr parser;
-	MgdObject *mobject = NULL;
-	MgdObject *lobject = NULL;
+	MidgardObject *mobject = NULL;
+	MidgardObject *lobject = NULL;
 	MidgardReflectionProperty *mrp = NULL;
 	const gchar *linktype = NULL;
 
@@ -417,7 +417,7 @@ gboolean _nodes2object(GObject *object, xmlNode *node, gboolean force)
 					MIDGARD_DBOBJECT_CLASS(klass));
 	}
 
-	gpointer set_from_xml_func = MIDGARD_METADATA_GET_CLASS(object)->dbpriv->set_from_xml_node;
+	gpointer set_from_xml_func = MIDGARD_DBOBJECT_GET_CLASS(object)->dbpriv->set_from_xml_node;
 	if(set_from_xml_func != NULL) {
 		MIDGARD_DBOBJECT_GET_CLASS(object)->dbpriv->set_from_xml_node(MIDGARD_DBOBJECT(object), node);
 		return TRUE;
@@ -427,7 +427,7 @@ gboolean _nodes2object(GObject *object, xmlNode *node, gboolean force)
 		if (cur->type == XML_ELEMENT_NODE) {
 		
 			linktype = NULL;
-
+	
 			GParamSpec *pspec = g_object_class_find_property(
 					G_OBJECT_GET_CLASS(G_OBJECT(object)), 
 					(const gchar *)cur->name);
@@ -464,21 +464,21 @@ gboolean _nodes2object(GObject *object, xmlNode *node, gboolean force)
 					}
 
 					/* we can use nodeprop directly */
-					lobject = midgard_object_class_get_object_by_guid(
-							mobject->dbpriv->mgd,
+					lobject = midgard_schema_object_factory_get_object_by_guid (
+							MIDGARD_DBOBJECT (mobject)->dbpriv->mgd,
 							(const gchar *) nodeprop);
 
 					if(!lobject && !force){
 						g_object_unref(mrp);
 						g_value_unset(&pval);
-						midgard_set_error(mobject->dbpriv->mgd, 
+						midgard_set_error(MGD_OBJECT_CNC (mobject), 
 								MGD_GENERIC_ERROR, 
 								MGD_ERR_MISSED_DEPENDENCE, 
 								" Can not import %s. "
 								"No '%s' object identified by '%s'",
 								G_OBJECT_TYPE_NAME(object),
 								linktype, nodeprop); 
-						g_clear_error(&mobject->dbpriv->mgd->err);	
+						g_clear_error(&MIDGARD_DBOBJECT (mobject)->dbpriv->mgd->err);	
 						return FALSE;
 					}
 					
@@ -533,7 +533,7 @@ gboolean _nodes2object(GObject *object, xmlNode *node, gboolean force)
 				}
 
 				set_property_unchecked:
-				switch (pspec->value_type) {
+				switch (G_TYPE_FUNDAMENTAL (pspec->value_type)) {
 				
 					case G_TYPE_STRING:
 						parser = xmlNewParserCtxt();
@@ -575,7 +575,7 @@ gboolean _nodes2object(GObject *object, xmlNode *node, gboolean force)
 						g_object_get(G_OBJECT(object),
 								(const gchar *) cur->name,
 								&prop_object, NULL);
-						_nodes2object(prop_object, cur->children, force);
+						_nodes2object(prop_object, cur->children, force);	
 						g_value_set_object(&pval, prop_object);
 						break;
 
@@ -665,7 +665,7 @@ GObject **midgard_core_object_from_xml(MidgardConnection *mgd,
 	}
 
 	guint _n_nodes = 0;
-	MgdObject **_objects = NULL;
+	MidgardObject **_objects = NULL;
 	GSList *olist = NULL;
 
 	for(; child; child = _get_type_node(child->next)) {
@@ -713,7 +713,7 @@ GObject **midgard_core_object_from_xml(MidgardConnection *mgd,
 					GValue strval = {0, };
 					g_value_init(&strval, G_TYPE_STRING);
 					g_value_set_string(&strval, purged);
-					midgard_core_metadata_set_revised(MIDGARD_OBJECT(object)->metadata, (const GValue*)&strval);
+					midgard_core_metadata_set_revised(MGD_DBOBJECT_METADATA(object), (const GValue*)&strval);
 					g_value_unset(&strval);
 				}
 				
@@ -750,7 +750,7 @@ GObject **midgard_core_object_from_xml(MidgardConnection *mgd,
 
 	if(_n_nodes > 0) {
 		
-		_objects = g_new(MgdObject *, _n_nodes+1);
+		_objects = g_new(MidgardObject *, _n_nodes+1);
 		_n_nodes = 0;
 		
 		for(; olist ; olist = olist->next){
@@ -767,3 +767,91 @@ GObject **midgard_core_object_from_xml(MidgardConnection *mgd,
 
 	return (GObject **) _objects;
 }
+
+/* Returns TRUE if property is private and caller is not allowed to read/write it. */ 
+gboolean 
+midgard_core_object_property_refuse_private (MidgardConnection *mgd, MgdSchemaTypeAttr *type, 
+		MidgardDBObject *object, const gchar *property)
+{
+	/* Unknown case, no connection  */
+	g_return_val_if_fail (mgd != NULL, TRUE);
+	g_return_val_if_fail (property != NULL, FALSE);
+
+	MgdSchemaTypeAttr *type_attr = type;
+	MidgardDBObjectClass *klass = NULL;
+
+	if (object)
+		klass = MIDGARD_DBOBJECT_GET_CLASS (object);
+
+	/* TODO, check if it's valid for cases like collector usage */
+	/* g_return_val_if_fail (object != NULL && type_attr != NULL, FALSE); */
+
+	if (!type_attr && object)
+		type_attr = klass->dbpriv->storage_data;
+
+	/* Unknown case, no storage_data which means something like no property */
+	g_return_val_if_fail (type_attr != NULL, FALSE);
+
+	MgdSchemaPropertyAttr *attr = midgard_core_schema_type_property_lookup (type_attr, property);
+	if (!attr)
+		return FALSE;
+
+	if (!attr->is_private)
+		return FALSE;
+
+	/* No object, so no chance to check metadata.creator.
+	 * There might be corner case or feature work around, so refuse unconditionally. */
+	if (!object) {
+		MIDGARD_ERRNO_SET_STRING (mgd, MGD_ERR_ACCESS_DENIED, "No object. Can not check private property owner");
+		return TRUE;
+	}
+
+	/* Also, no user, and anonymous is not allowed.
+	 * If user exists, and has no person associated, refuse as well. */ 
+	MidgardObject *person = MGD_CNC_PERSON (mgd);
+	if (!mgd->priv->user) {
+		MIDGARD_ERRNO_SET_STRING (mgd, MGD_ERR_ACCESS_DENIED, "No user logged in");
+		return TRUE;
+	}
+
+	if (!person) {
+		MIDGARD_ERRNO_SET_STRING (mgd, MGD_ERR_ACCESS_DENIED, "Can not check property owner. No person associated with logged in user");
+		return TRUE;
+	}
+
+	/* OK, so we have admin, allow */
+	if (midgard_user_is_admin (mgd->priv->user))
+		return FALSE;
+
+	const gchar *person_guid = MGD_OBJECT_GUID (person);
+
+	/* Something really strange, person has no guid associated */
+	if (!person_guid) {
+		MIDGARD_ERRNO_SET_STRING (mgd, MGD_ERR_ACCESS_DENIED, "Midgard person identified by empty guid");
+		return TRUE;
+	}
+
+	gchar *creator_guid = NULL;
+
+	if (MIDGARD_IS_OBJECT (object)) {
+		MidgardMetadata *metadata = MGD_DBOBJECT_METADATA (object);
+		creator_guid = metadata->priv->creator;
+	}
+
+	/* TODO, sooner or later we will need metadata->owner check */
+
+	/* No creator for this object. Might be very old in db or not set at all.
+	 * Refuse, we do not know this. */
+	if (!creator_guid) {
+		MIDGARD_ERRNO_SET_STRING (mgd, MGD_ERR_ACCESS_DENIED, "Object's metadata holds null value instead of creator guid");
+		return TRUE;
+	}
+	
+	/* Finally, user which is logged in is the same person which created 
+	 * object in question. Allow. */
+	if (g_str_equal (person_guid, creator_guid))
+		return FALSE;
+
+	MIDGARD_ERRNO_SET (mgd, MGD_ERR_ACCESS_DENIED);
+	return TRUE;
+}
