@@ -34,22 +34,48 @@ struct _MidgardDBObjectPrivate {
 	MgdSchemaTypeAttr *storage_data;
 	MidgardConnection *mgd;
 	gboolean has_metadata;
+	MidgardMetadata *metadata;
 	GdaDataModel *datamodel;
-	gint row;	
+	gint row;
+
+	/* GDA pointers */
+	GdaStatement *statement_insert;
+	GdaSet *param_set_insert;	
+
+	/* GdaSql virtual helpers */
+	void			(*add_fields_to_select_statement)	(MidgardDBObjectClass *klass, 
+			GdaSqlStatementSelect *select, const gchar *table_name);
 
 	GSList 			*(*set_from_sql)	(MidgardConnection *mgd, GType type, const gchar *sql);	
 	void 			(*__set_from_sql)	(MidgardDBObject *self, GdaDataModel *model, gint row);
+	void 			(*set_from_data_model)	(MidgardDBObject *self, GdaDataModel *model, gint row);
 	void 			(*set_from_xml_node)	(MidgardDBObject *self, xmlNode *node);
 	MidgardConnection 	*(*get_connection) 	(MidgardDBObject *self);
 	gboolean		(*create_storage)	(MidgardConnection *mgd, MidgardDBObjectClass *klass);
 	gboolean 		(*update_storage)	(MidgardConnection *mgd, MidgardDBObjectClass *klass);
 	gboolean		(*storage_exists)	(MidgardConnection *mgd, MidgardDBObjectClass *klass);
 	gboolean		(*delete_storage)	(MidgardConnection *mgd, MidgardDBObjectClass *klass);
+	gboolean		(*get_property)		(MidgardDBObject *self, const gchar *name, GValue *value);
+	gboolean		(*set_property)		(MidgardDBObject *self, const gchar *name, GValue *value);
+
+	/* GDA helpers */
+	void			(*set_statement_insert)	(MidgardDBObjectClass *klass);
 };
 
 #define MGD_OBJECT_GUID(___obj) MIDGARD_DBOBJECT(___obj)->dbpriv->guid
 #define MGD_OBJECT_CNC(___obj) MIDGARD_DBOBJECT(___obj)->dbpriv->mgd
 
+#define MGD_DBOBJECT_DBPRIV (__obj) MIDGARD_DBOBJECT(__obj)->dbpriv
+#define MGD_DBOBJECT_HAS_METADATA (__obj) MGD_TYPE_ATTR_METADATA_CLASS(MGD_DBOBJECT_DBPRIV(__obj)->storage_data) ? TRUE : FALSE; 
+#define MGD_DBOBJECT_METADATA(___obj) MIDGARD_DBOBJECT(___obj)->dbpriv->metadata
+#define MGD_DBCLASS_METADATA_CLASS(__klass) MIDGARD_DBOBJECT_CLASS(__klass)->dbpriv->storage_data->metadata_class_ptr
+#define MGD_DBCLASS_PROPERTY_UP(__klass) MIDGARD_DBOBJECT_CLASS(__klass)->dbpriv->storage_data->property_up
+#define MGD_DBCLASS_PROPERTY_PARENT(__klass) MIDGARD_DBOBJECT_CLASS(__klass)->dbpriv->storage_data->property_parent
+#define MGD_DBCLASS_PROPERTY_UNIQUE(__klass) MIDGARD_DBOBJECT_CLASS(__klass)->dbpriv->storage_data->unique_name
+
+#define MGD_DBCLASS_PRIMARY(__klass) MIDGARD_DBOBJECT_CLASS(__klass)->dbpriv->storage_data->primary;
+#define MGD_DBCLASS_TABLENAME(__klass) MIDGARD_DBOBJECT_CLASS(__klass)->dbpriv->storage_data->table;
+
 /* Private structure for private data of MgdSchema objects */
 struct _MidgardObjectPrivate{
 	const gchar *action;
@@ -98,7 +124,7 @@ struct _MidgardMetadataPrivate {
 	gboolean approve_is_set;
 	
 	/* Other sruct members */
-	MgdObject *object;
+	MidgardDBObject *object;
 };
 
 /* Private structure for Midgard Config object */
@@ -129,13 +155,6 @@ struct _MidgardConfigPrivate{
 	gchar *pamfile;
 };
 
-typedef struct _MidgardConnectionCache MidgardConnectionCache;
-
-struct _MidgardConnectionCache {
-	GData *sg_datalist;
-	gpointer sg_ids;
-};
-
 struct _MidgardConnectionPrivate {
 	MidgardConnection *pattern; /* a pointer to connection we made copy from */
         MidgardConfig *config;
@@ -154,13 +173,11 @@ struct _MidgardConnectionPrivate {
 
 	const gchar *cnc_str;
 	const gchar *auth_str;
-
-	GObject *person;
-	GObject *user;
+	
+	MidgardUser *user;
 	gboolean inherited;
 	GTimeVal app_time;
 	GTimer *timer;
-	MidgardConnectionCache *cache;
 	gchar *configname;
 
 	gboolean error_clbk_connected;
@@ -168,10 +185,21 @@ struct _MidgardConnectionPrivate {
 	/* Auth stack & types */
 	GSList *authtypes;
 	GSList *authstack;
+
+	gboolean enable_replication;
+	gboolean enable_quota;
+	gboolean enable_debug;
+	gboolean enable_dbus;
 };
 
+#define MGD_CNC_PERSON(_cnc) _cnc->priv->user ? midgard_user_get_person (_cnc->priv->user) : NULL
+#define MGD_CNC_QUOTA(_cnc) _cnc->priv->enable_quota
+#define MGD_CNC_REPLICATION(_cnc) _cnc->priv->enable_replication
+#define	MGD_CNC_DEBUG(_cnc) _cnc->priv->enable_debug
+#define MGD_CNC_DBUS(_cnc) _cnc->priv->enable_dbus
+
 struct _MidgardBlobPrivate {
-	MgdObject *attachment;
+	MidgardObject *attachment;
 	gchar *location;
 	MidgardConnection *mgd;
 	gchar *blobdir;
@@ -182,10 +210,6 @@ struct _MidgardBlobPrivate {
 	gchar *encoding;
 };
 
-struct _MidgardSitegroupPrivate {
-	gchar *initial_name;
-};
-
 typedef enum {
 	OBJECT_UPDATE_NONE = 0,
 	OBJECT_UPDATE_EXPORTED,
@@ -193,6 +217,9 @@ typedef enum {
 } _ObjectActionUpdate;
 
 
+/* MidgardDBObject */
+GParamSpec **midgard_core_dbobject_class_list_properties (MidgardDBObjectClass *klass, guint *n_props); 
+
 /* Object's xml */
 xmlDoc *midgard_core_object_create_xml_doc(void);
 void midgard_core_object_get_xml_doc(  MidgardConnection *mgd,
@@ -205,8 +232,8 @@ xmlNode *_get_type_node(xmlNode *node);
 GObject **midgard_core_object_from_xml(MidgardConnection *mgd, const gchar *xml, gboolean force);
 
 /* Object's routines */
-gboolean _midgard_object_update(MgdObject *object, _ObjectActionUpdate replicate);
-gboolean _midgard_object_create(MgdObject *object, const gchar *create_guid, _ObjectActionUpdate replicate);
+gboolean _midgard_object_update(MidgardObject *object, _ObjectActionUpdate replicate);
+gboolean _midgard_object_create(MidgardObject *object, const gchar *create_guid, _ObjectActionUpdate replicate);
 void _object_copy_properties(GObject *src, GObject *dest);
 
 /* Links */
@@ -218,12 +245,15 @@ gboolean midgard_core_object_prop_parent_is_valid(GType ptype);
 gboolean midgard_core_object_prop_up_is_valid(GType ptype);
 GType midgard_core_object_get_property_parent_type(MidgardObjectClass *klass);
 GType midgard_core_object_get_property_up_type(MidgardObjectClass *klass);
-gboolean midgard_core_object_prop_parent_is_set(MgdObject *object);
-gboolean midgard_core_object_prop_up_is_set(MgdObject *object);
+gboolean midgard_core_object_prop_parent_is_set(MidgardObject *object);
+gboolean midgard_core_object_prop_up_is_set(MidgardObject *object);
 
-gboolean midgard_core_object_has_dependents(MgdObject *self, const gchar *classname);
+gboolean midgard_core_object_has_dependents(MidgardObject *self, const gchar *classname);
+
+/* ACL */
+gboolean midgard_core_object_property_refuse_private (MidgardConnection *self, MgdSchemaTypeAttr *attr, MidgardDBObject *object, const gchar *property);
 
 /* D-Bus */
-void midgard_core_dbus_send_serialized_object(MgdObject *object, const gchar *path);
+void midgard_core_dbus_send_serialized_object(MidgardObject *object, const gchar *path);
 
 #endif /* MIDGARD_CORE_OBJECT_H */
