@@ -23,6 +23,10 @@
 #include "midgard_dbobject.h"
 #include <libgda/libgda.h>
 #include "schema.h"
+#include "midgard_query_property.h"
+#include "midgard_query_storage.h"
+#include "midgard_query_constraint_simple.h"
+#include "midgard_query_executor.h"
 
 typedef struct _MidgardDBColumn MidgardDBColumn;
 typedef struct _MidgardDBJoin MidgardDBJoin;
@@ -52,6 +56,49 @@ struct _MidgardDBJoin {
 	MgdSchemaPropertyAttr *right;
 };
 
+struct MidgardCoreQueryOrder {
+	MidgardQueryProperty *property;
+	const gchar *type;
+};
+
+struct _MidgardQueryStoragePrivate {
+	MidgardDBObjectClass *klass;
+	gchar *table_alias;
+	const gchar *table;
+};
+
+struct _MidgardQueryPropertyPrivate {
+	GValue value;
+	MidgardDBObjectClass *klass;
+	MidgardQueryStorage *storage;
+};
+
+struct _MidgardQueryExecutorPrivate {
+	MidgardConnection *mgd;
+	MidgardQueryStorage *storage;
+	MidgardQueryConstraintSimple *constraint;
+	GSList *orders;
+	GSList *joins;
+	gint limit;
+	gint offset;
+	gpointer resultset;
+	guint tableid;
+	gchar *table_alias;
+	guint joinid;
+	GdaSqlStatement *stmt;
+	guint results_count;
+	gboolean read_only;
+};
+
+struct _MidgardQueryConstraintSimplePrivate {
+	void	(*add_conditions_to_statement)	(MidgardQueryExecutor *executor, MidgardQueryConstraintSimple *self, 
+			GdaSqlStatement *stm, GdaSqlExpr *where_expr_node);
+};
+
+#define MQE_SET_TABLE_ALIAS(executor, table) \
+	if (!table) \
+		table = g_strdup_printf ("t%d", ++executor->priv->tableid);
+
 MidgardDBJoin	*midgard_core_dbjoin_new	(void);
 void		midgard_core_dbjoin_free	(MidgardDBJoin *mdbj);
 
@@ -212,7 +259,8 @@ gboolean midgard_core_table_exists(
 GdaDataModel 		*midgard_core_query_get_dbobject_model 		(MidgardConnection *mgd, MidgardDBObjectClass *klass, guint n_params, const GParameter *parameters);
 gboolean		midgard_core_query_create_dbobject_record 	(MidgardDBObject *object);
 gboolean		midgard_core_query_update_dbobject_record 	(MidgardDBObject *object);
-gchar 			*midgard_core_query_binary_stringify		(GValue *src_value); 
-gchar 			*midgard_core_query_escape_string		(MidgardConnection *mgd, const gchar *str);
+gchar                   *midgard_core_query_binary_stringify            (GValue *src_value);
+gchar 			*midgard_core_query_compute_constraint_property	(MidgardQueryExecutor *executor, MidgardQueryStorage *storage, const gchar *name);
+
 
 #endif /* MIDGARD_CORE_QUERY_H */
