@@ -27,11 +27,12 @@ typedef struct {
 
 static GHashTable *__copy_hash_table_with_strings(GHashTable *src);
 
-MgdSchemaPropertyAttr* 
+MgdSchemaPropertyAttr *  
 midgard_core_schema_type_property_attr_new_empty (void)
 {
 	MgdSchemaPropertyAttr *prop = g_new(MgdSchemaPropertyAttr, 1); 
 
+	/* Ensure, default type is string */
 	prop->gtype = G_TYPE_NONE;
 	prop->type = NULL;
 
@@ -51,23 +52,25 @@ midgard_core_schema_type_property_attr_new_empty (void)
 	prop->is_reversed = FALSE;
 	prop->is_link = FALSE;
 	prop->is_linked = FALSE;
+	prop->is_private = FALSE;
+	prop->is_unique = FALSE;
 	prop->description = NULL;
-	prop->user_values = NULL; 
+	prop->user_values = NULL;
 
 	return prop;
 }
 
-MgdSchemaPropertyAttr *
+MgdSchemaPropertyAttr * 
 midgard_core_schema_type_property_attr_new (void)
 {
 	MgdSchemaPropertyAttr *prop = midgard_core_schema_type_property_attr_new_empty();
 
-	/* Ensure default type is string */
+	/* Ensure, default type is string */
 	prop->gtype = G_TYPE_STRING;
-	prop->type = g_strdup("string");
+	prop->type = g_strdup ("string");
 
-	prop->description = g_strdup("");
-	prop->user_values = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
+	prop->description = g_strdup ("");
+	prop->user_values = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
 
 	return prop;
 }
@@ -194,7 +197,8 @@ midgard_core_schema_type_attr_new (void)
 	type->constraints = NULL;
 	type->is_view = FALSE;
 	type->sql_create_view = NULL;
-	type->metadata_class = NULL;
+	type->metadata_class_name = NULL;
+	type->metadata_class_ptr = NULL;
 	type->user_values = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
 
 	return type;
@@ -219,9 +223,10 @@ midgard_core_schema_type_attr_extend (MgdSchemaTypeAttr *src, MgdSchemaTypeAttr
 			dst->children = g_slist_append (dst->children, g_strdup ((gchar *) children->data));
 		}
 	}
-	
+
 	dst->unique_name = src->unique_name ? g_strdup (src->unique_name) : NULL;
-	dst->metadata_class = src->metadata_class ? g_strdup (src->metadata_class) : NULL;
+	dst->metadata_class_name = src->metadata_class_name ? g_strdup (src->metadata_class_name) : NULL;
+	dst->metadata_class_ptr = src->metadata_class_ptr ? src->metadata_class_ptr : NULL;
 }
 
 void 
@@ -297,8 +302,8 @@ midgard_core_schema_type_attr_free (MgdSchemaTypeAttr *type)
 
 	g_free (type->sql_create_view);
 	type->sql_create_view = NULL;
-	g_free (type->metadata_class);
-	type->metadata_class = NULL;
+	g_free (type->metadata_class_name);
+	type->metadata_class_name = NULL;
 
 	g_hash_table_destroy (type->user_values);
         type->user_values = NULL;
@@ -490,7 +495,7 @@ midgard_core_schema_type_property_set_tablefield (MgdSchemaPropertyAttr *prop, c
 		g_free ((gchar *)prop->tablefield);
 
 	gchar *_table = g_strdup (table);
-	gchar *_field = g_strdup (field);
+	gchar *_field = g_ascii_strdown (field, -1);
 
 	if (prop->table) g_free ( (gchar *)prop->table);
 	prop->table = g_strdup(_table);
@@ -603,17 +608,17 @@ midgard_core_schema_type_build_static_sql (MgdSchemaTypeAttr *type_attr)
 		return;
 
 	GString *_sql = g_string_new(" ");
-	_str_cont *cont = g_new(_str_cont, 1);
-	cont->string = _sql;
-	cont->elts = 0;
-	cont->type = type_attr;
+	_str_cont cont = {NULL, 0, NULL};
+	cont.string = _sql;
+	cont.elts = 0;
+	cont.type = type_attr;
 
-	g_hash_table_foreach (type_attr->prophash, __build_static_sql, cont);
+	g_hash_table_foreach (type_attr->prophash, __build_static_sql, &cont);
 
 	if (_sql->str == NULL)
 		return;
 
-	type_attr->sql_select_full = g_strdup(_sql->str);
+	type_attr->sql_select_full = g_string_free (_sql, FALSE);
 }
 
 static void 
@@ -659,5 +664,8 @@ midgard_core_schema_type_property_lookup (MgdSchemaTypeAttr *type, const gchar *
 	g_return_val_if_fail (type != NULL, NULL);
 	g_return_val_if_fail (name != NULL, NULL);
 
+	if (!type->prophash)
+		return NULL;
+
 	return g_hash_table_lookup (type->prophash, name);
 }
