@@ -31,7 +31,6 @@
 #include "midgard_core_query.h"
 #include "query_constraint.h"
 #include "midgard_core_views.h"
-#include "midgard_object_class.h"
 #include "midgard_metadata.h"
 
 #define MGD_VIEW_RW_VIEW "view"
@@ -64,16 +63,13 @@ static void __get_property_attribute (xmlNode *node, gchar **property_string,
 	/* Check metadata */
 	if (property_string[2] != NULL && g_str_equal (*target_name, "metadata")) {
 
-		if (!midgard_object_class_has_metadata (MIDGARD_OBJECT_CLASS (klass))) {
+		if (!MGD_DBCLASS_METADATA_CLASS (klass)) {
 			__view_error (node, "No metadata registered for %s class", G_OBJECT_CLASS_NAME (klass));
 			return;
 		}
 
-		MidgardMetadataClass *metadata_klass = 
-			MIDGARD_METADATA_CLASS (midgard_object_class_get_metadata_class (MIDGARD_OBJECT_CLASS (klass)));
-
 		*target_name = property_string[2];
-		*prop_attr = g_hash_table_lookup (metadata_klass->dbpriv->storage_data->prophash, *target_name);
+		*prop_attr = g_hash_table_lookup (MIDGARD_DBOBJECT_CLASS (klass)->dbpriv->storage_data->prophash, *target_name);
 	
 		if (!*prop_attr)
 			__view_error (node, "%s not found. Not registered for %s.metadata ?", *target_name, property_string[0]);
@@ -84,6 +80,10 @@ static void __get_property_attribute (xmlNode *node, gchar **property_string,
 		*prop_attr = g_hash_table_lookup (klass->dbpriv->storage_data->prophash, *target_name);
 		if (!*prop_attr)
 			__view_error(node, "%s not found. Not registered for %s ?", *target_name, property_string[0]);
+
+		if ((*prop_attr)->is_private)
+			__view_error (node, "Private property %s.%s can not be added to view.",
+					G_OBJECT_CLASS_NAME (G_OBJECT_CLASS (klass)), (*prop_attr)->name);
 	}
 }
 
@@ -112,6 +112,7 @@ static void __get_view_properties(xmlNode *node, MgdSchemaTypeAttr *type)
 				__view_error(cur, "Referenced class:property can not be empty", NULL);
 
 			gchar **rprop = g_strsplit_set ((const gchar *)use_prop, ":.", -1);
+			xmlFree (use_prop);
 			if (!rprop || rprop[0] == NULL || rprop[1] == NULL) {
 				__view_error(cur, "Referenced property can not be empty", NULL);
 				return;
@@ -126,7 +127,7 @@ static void __get_view_properties(xmlNode *node, MgdSchemaTypeAttr *type)
 				__view_error (cur, "Can not create proper view. Defined '%s' class has NULL storage", rprop[0]);
 
 			__get_property_attribute (cur, rprop, &property_name, &rprop_attr, MIDGARD_DBOBJECT_CLASS (klass));
-	
+
 			midgard_core_schema_type_property_copy(rprop_attr, type);
 			
 			/* TODO, refactor with some usable TypeAttr related API */
@@ -165,8 +166,7 @@ static const gchar *__allowed_joins[] = {"left", "right", "inner", "inner left",
 static void __get_view_joins(xmlNode *node, MgdSchemaTypeAttr *type)
 {
 	xmlNode *cur;
-	gchar *property_name = NULL;
-	MgdSchemaPropertyAttr *prop_attr = NULL;
+	gchar *property_name = NULL;	
 	MgdSchemaPropertyAttr *propright = NULL;
 	MgdSchemaPropertyAttr *propleft = NULL;
 
@@ -308,8 +308,8 @@ static void __get_view_constraints(xmlNode *node, MgdSchemaTypeAttr *type)
 			if (!klass)
 				__view_error(cur, "Class %s not registered", classprop[0]);
 
-			MidgardQueryConstraint *constraint = midgard_query_constraint_new();
-			if (!midgard_query_constraint_parse_property(&constraint, klass, classprop[1]))
+			MidgardCoreQueryConstraint *constraint = midgard_core_query_constraint_new();
+			if (!midgard_core_query_constraint_parse_property(&constraint, klass, classprop[1]))
 				__view_error(cur, "Can not parse %s constraint property", classprop[1]);
 
 			g_strfreev(classprop);
@@ -319,7 +319,7 @@ static void __get_view_constraints(xmlNode *node, MgdSchemaTypeAttr *type)
 			xmlChar *operator = xmlGetProp(cur, (const xmlChar *)"operator");
 			if (!operator || (operator && *operator == '\0'))
 				__view_error(cur, "Can not add constraint with empty operator", NULL);
-			if (!midgard_query_constraint_add_operator(constraint, (const gchar *)operator))
+			if (!midgard_core_query_constraint_add_operator(constraint, (const gchar *)operator))
 				__view_error(cur, "Invalid operator", NULL);
 			xmlFree(operator);
 
@@ -347,8 +347,8 @@ static void __get_view_constraints(xmlNode *node, MgdSchemaTypeAttr *type)
 			g_value_init(&val, vtype);
 			g_value_transform((const GValue *) &strval, &val);
 		
-			midgard_query_constraint_add_value(constraint, &val);
-			midgard_query_constraint_build_condition(constraint);
+			midgard_core_query_constraint_add_value(constraint, &val);
+			midgard_core_query_constraint_build_condition(constraint);
 
 			type->constraints = g_slist_append(type->constraints, (gpointer) constraint);
 			
@@ -450,7 +450,7 @@ gchar *__build_static_create_view(MgdSchemaTypeAttr *type)
 		
 		g_string_append(query, " AND ");
 		g_string_append(query,
-				MIDGARD_QUERY_CONSTRAINT(list->data)->priv->condition);
+				MIDGARD_CORE_QUERY_CONSTRAINT(list->data)->priv->condition);
 	}
 
 	return g_string_free(query, FALSE);
