@@ -24,11 +24,15 @@
 #include "midgard_metadata.h"
 #include "midgard_user.h"
 
-gboolean midgard_quota_size_is_reached(MgdObject *object, gint size)
+gboolean midgard_quota_size_is_reached(MidgardObject *object, gint size)
 {
 	GString *query;
 	guint limit_tmp_size = 0, tmp_size = 0;	
 	MidgardConnection *mgd = MGD_OBJECT_CNC (object);
+
+	if (!MGD_CNC_QUOTA (mgd))
+		return FALSE;
+
 	guint rows = 0;
 	const GValue *tmp_size_value;
 	const GValue *limit_tmp_size_value;	
@@ -91,9 +95,13 @@ gboolean midgard_quota_size_is_reached(MgdObject *object, gint size)
 /*
  * Returns object's disk usage size 
  */ 
-guint midgard_quota_get_object_size(MgdObject *object)
+guint midgard_quota_get_object_size(MidgardObject *object)
 {
 	g_assert(object != NULL);
+	MidgardConnection *mgd = MGD_OBJECT_CNC (object);
+
+	if (!MGD_CNC_QUOTA (mgd))
+		return 0;
 
 	const gchar *tablename = 
 		midgard_core_class_get_table(MIDGARD_DBOBJECT_GET_CLASS(object));
@@ -127,20 +135,24 @@ guint midgard_quota_get_object_size(MgdObject *object)
  * Updates object's storage setting object's size
  * WARNING, this is MySQL optimized!	
  */
-gboolean midgard_quota_update(MgdObject *object, guint32 init_size)
+gboolean midgard_quota_update(MidgardObject *object, guint32 init_size)
 {
 	g_assert(object != NULL);
-	g_assert(MGD_OBJECT_CNC (object) != NULL);
+	MidgardConnection *mgd = MGD_OBJECT_CNC (object);
+	g_assert(mgd != NULL);
+
+	if (!MGD_CNC_QUOTA (mgd))
+		return TRUE;
 
 	guint32 size = 0;
 	guint32 qsize = 0;
 	const gchar *typename = G_OBJECT_TYPE_NAME(object);	
 	gchar *tmpstr;
 
-	if (!object->metadata)
+	if (!MGD_DBOBJECT_METADATA (object))
 		return TRUE;
 
-	size = object->metadata->priv->size;
+	size = MGD_DBOBJECT_METADATA (object)->priv->size;
 	
 	/* nothing to update, this should never happen */
 	if(size == 0) return TRUE;
@@ -173,7 +185,7 @@ gboolean midgard_quota_update(MgdObject *object, guint32 init_size)
 	return TRUE;
 }
 
-gboolean midgard_quota_create(MgdObject *object)
+gboolean midgard_quota_create(MidgardObject *object)
 {
 	g_assert(object != NULL);
 
@@ -181,11 +193,14 @@ gboolean midgard_quota_create(MgdObject *object)
 	MidgardConnection *mgd = MGD_OBJECT_CNC (object);
 	gchar *tmpstr;
 
-	/* Bit of hack, but there's no hook to store object's size */
-	if (!object->metadata)
+	if (!MGD_CNC_QUOTA (mgd))
 		return TRUE;
 
-	guint32 size = object->metadata->priv->size;
+	/* Bit of hack, but there's no hook to store object's size */
+	if (!MGD_DBOBJECT_METADATA (object))
+		return TRUE;
+
+	guint32 size = MGD_DBOBJECT_METADATA (object)->priv->size;
 	if(midgard_quota_size_is_reached(object, size)){
 		MIDGARD_ERRNO_SET(mgd, MGD_ERR_QUOTA);
 		return FALSE;
@@ -204,10 +219,15 @@ gboolean midgard_quota_create(MgdObject *object)
 	return TRUE;
 }
 
-void midgard_quota_remove(MgdObject *object, guint size){
+void midgard_quota_remove(MidgardObject *object, guint size){
 
 	g_assert(object != NULL);
 
+	MidgardConnection *mgd = MGD_OBJECT_CNC (object);
+
+	if (!MGD_CNC_QUOTA (mgd))
+		return;
+
 	GString *query;
 	gchar *tmpstr;
 	const gchar *typename = G_OBJECT_TYPE_NAME(object);
