@@ -49,13 +49,13 @@ gboolean        	__midgard_user_update             (MidgardUser *self);
 gboolean		__midgard_user_delete		  (MidgardUser *self);
 gboolean        	__midgard_user_is_user            (MidgardUser *self);
 gboolean        	__midgard_user_is_admin           (MidgardUser *self);
-MgdObject		*__midgard_user_get_person        (MidgardUser *self);
+MidgardObject		*__midgard_user_get_person        (MidgardUser *self);
 gboolean		__midgard_user_set_person         (MidgardUser *self, MidgardObject *person);
 gboolean        	__midgard_user_login              (MidgardUser *self);
 gboolean        	__midgard_user_logout             (MidgardUser *self);
 
 struct _MidgardUserPrivate {
-	MgdObject *person;
+	MidgardObject *person;
 	gchar *person_guid;
 	guint user_type;
 	gchar *login;
@@ -84,28 +84,6 @@ enum {
 	MIDGARD_USER_PERSON_GUID
 };
 
-static gchar *
-__string2md5hash (const gchar *str) 
-{
-	g_assert(str != NULL);
-
-	guchar mdbuf[16];
-
-	MIDGARD_MD5_CTX md5;
-	MIDGARD_MD5_Init(&md5);
-	MIDGARD_MD5_Update(&md5, (unsigned char *) str, strlen(str));
-	MIDGARD_MD5_Final(mdbuf, &md5);
-	int i;
-	
-	GString *gstr = g_string_new("");
-	
-	for (i = 0; i < 16; i++) {
-		g_string_append_printf(gstr, "%02x", mdbuf[i]);	
-	}
-
-	return g_string_free(gstr, FALSE);
-}
-
 static GObjectClass *__parent_class= NULL;
 
 /**
@@ -138,24 +116,6 @@ midgard_user_new (MidgardConnection *mgd, guint n_params, const GParameter *para
 	return self;
 }
 
-/**
- * midgard_user_auth:
- *
- * Deprecated: 9.09
- * See: midgard_user_login() 
- */
-MidgardUser *
-midgard_user_auth (MidgardConnection *mgd, 
-		const gchar *name, const gchar *password, 
-		const gchar *sitegroup, gboolean trusted)
-{
-	g_return_val_if_fail(mgd != NULL, NULL);
-
-	MIDGARD_ERRNO_SET(mgd, MGD_ERR_INTERNAL);
-
-	return NULL;
-}
-
 static gboolean 
 __validate_parameters (MidgardConnection *mgd, guint n_params, const GParameter *parameters, guint valid)
 {
@@ -309,7 +269,7 @@ __midgard_user_get (MidgardConnection *mgd, guint n_params, const GParameter *pa
 	MidgardUser *self = g_object_new (MIDGARD_TYPE_USER, NULL);
 	MGD_OBJECT_CNC (self) = mgd;
 
-	klass->dbpriv->__set_from_sql (MIDGARD_DBOBJECT (self), model, 0);
+	MIDGARD_DBOBJECT_CLASS (klass)->dbpriv->__set_from_sql (MIDGARD_DBOBJECT (self), model, 0);
 
 	return self;
 }
@@ -410,7 +370,7 @@ __midgard_user_query (MidgardConnection *mgd, guint n_params, const GParameter *
 	
 		users[i] = g_object_new (MIDGARD_TYPE_USER, NULL);
 		MIDGARD_DBOBJECT (users[i])->dbpriv->mgd = mgd;
-		klass->dbpriv->__set_from_sql (MIDGARD_DBOBJECT (users[i]), model, i);
+		MIDGARD_DBOBJECT_CLASS (klass)->dbpriv->__set_from_sql (MIDGARD_DBOBJECT (users[i]), model, i);
 	}
 
 	users[i] = NULL;
@@ -529,13 +489,13 @@ __midgard_user_create (MidgardUser *self)
 	/* Set correct auth type id */
 
 	/* Create record */
-	self->dbpriv->guid = midgard_guid_new (mgd);
+	MIDGARD_DBOBJECT (self)->dbpriv->guid = midgard_guid_new (mgd);
 
 	if (midgard_core_query_create_dbobject_record (MIDGARD_DBOBJECT (self)))
 		return TRUE;
 
-	g_free ( (gchar *)self->dbpriv->guid);
-	self->dbpriv->guid = NULL;
+	g_free ( (gchar *)MIDGARD_DBOBJECT (self)->dbpriv->guid);
+	MIDGARD_DBOBJECT (self)->dbpriv->guid = NULL;
 
 	MIDGARD_ERRNO_SET (mgd, MGD_ERR_INTERNAL);
 
@@ -728,25 +688,34 @@ __midgard_user_delete (MidgardUser *self)
 
 	GError *error = NULL;
 	GdaConnection *connection = mgd->priv->connection;
+
+#ifdef HAVE_LIBGDA_4
+	
+	gda_execute_non_select_command (connection, del_query->str, &error);
+
+#else 
 	GdaCommand *command = gda_command_new (del_query->str, GDA_COMMAND_TYPE_SQL, GDA_COMMAND_OPTION_STOP_ON_ERRORS);
 	gda_connection_execute_non_select_command (connection, command, NULL, &error);
 	gda_command_free (command);
-	g_string_free (del_query, TRUE);
+
+#endif /* HAVE_LIBGDA_4 */
 	
+	g_string_free (del_query, TRUE);
+
 	if(error){
 		g_clear_error(&error);
 		return FALSE;
 	}
 
 	/* Set empty guid */
-	g_free ((gchar *)self->dbpriv->guid);
-	self->dbpriv->guid = NULL;
+	g_free ((gchar *)MIDGARD_DBOBJECT (self)->dbpriv->guid);
+	MIDGARD_DBOBJECT (self)->dbpriv->guid = NULL;
 
 	return TRUE;
 }
 
 /** 
- * midgard_user_login:
+ * midgard_user_log_in:
  * @self: #MidgardUser instance
  * 
  * Logs in user instance, if given one is valid. 
@@ -766,12 +735,12 @@ __midgard_user_delete (MidgardUser *self)
  * Since: 9.09
  */
 gboolean 
-midgard_user_login (MidgardUser *self)
+midgard_user_log_in (MidgardUser *self)
 {
 	g_return_val_if_fail (self != NULL, FALSE);
 
 	MidgardUserClass *klass = MIDGARD_USER_GET_CLASS (self);
-	return klass->login (self);
+	return klass->log_in (self);
 }
 
 gboolean
@@ -795,7 +764,7 @@ __midgard_user_login (MidgardUser *self)
 	}
 
 	g_object_ref(self); // storing as priv->user
-	mgd->priv->user = G_OBJECT(self);
+	mgd->priv->user = self;
 
 	g_object_ref(self); // storing in priv->authstack
 	mgd->priv->authstack = g_slist_append(mgd->priv->authstack, self);
@@ -807,7 +776,7 @@ __midgard_user_login (MidgardUser *self)
 }
 
 /**
- * midgard_user_logout:
+ * midgard_user_log_out:
  * @self: #MidgardUser instance
  *
  * Cases to return %FALSE:
@@ -825,12 +794,12 @@ __midgard_user_login (MidgardUser *self)
  * Since: 9.09
  */
 gboolean 
-midgard_user_logout (MidgardUser *self)
+midgard_user_log_out (MidgardUser *self)
 {
 	g_return_val_if_fail(self != NULL, FALSE);
 
 	MidgardUserClass *klass = MIDGARD_USER_GET_CLASS(self);
-	return klass->logout(self);
+	return klass->log_out(self);
 }
 
 gboolean 
@@ -942,12 +911,12 @@ gboolean midgard_user_set_active(MidgardUser *self, gboolean flag)
 {
 	g_assert(self != NULL);
 
-	if(!self->dbpriv->mgd) {
+	if(!MIDGARD_DBOBJECT (self)->dbpriv->mgd) {
 		g_warning("Can not set activity for user without person assigned");
 		return FALSE;
 	}
 
-	MidgardConnection *mgd = self->dbpriv->mgd;
+	MidgardConnection *mgd = MIDGARD_DBOBJECT (self)->dbpriv->mgd;
 	MidgardUser *user = MIDGARD_USER(mgd->priv->user);
 
 	MIDGARD_ERRNO_SET (mgd, MGD_ERR_OK);
@@ -970,7 +939,7 @@ gboolean midgard_user_set_active(MidgardUser *self, gboolean flag)
 	g_string_append_printf(sql, 
 			"active = %d WHERE guid = '%s' ",
 			active,
-			self->dbpriv->guid); 
+			MIDGARD_DBOBJECT (self)->dbpriv->guid); 
 			
 	
 	gint rv =  midgard_core_query_execute(mgd, sql->str, FALSE);
@@ -989,234 +958,6 @@ gboolean midgard_user_set_active(MidgardUser *self, gboolean flag)
 	return TRUE;
 }
 
-static gboolean __set_legacy_password(MidgardUser *self, const gchar *login, 
-		const gchar *password, guint hashtype) 
-{
-	if(self->dbpriv->guid == NULL)
-		return FALSE;
-
-	GValue gval = {0, };
-	g_value_init(&gval, G_TYPE_STRING);
-	g_value_set_string(&gval, self->dbpriv->guid);
-	MgdObject *person = 
-		midgard_object_new(self->dbpriv->mgd, "midgard_person", &gval);
-	g_value_unset(&gval);
-
-	if(!person)
-		return FALSE;
-	
-	GString *sql = g_string_new("UPDATE person SET ");
-	g_string_append_printf(sql, "username = '%s', password = ", login);
-	
-	if(hashtype == MIDGARD_USER_HASH_LEGACY) {
-		
-		g_string_append_printf(sql, "Encrypt('%s') ", password);
-
-	} else {
-
-		g_string_append_printf(sql, "'**%s' ", password);
-	}
-
-	g_string_append_printf(sql, "WHERE guid = '%s' ", MGD_OBJECT_GUID (self));
-
-	gint up = midgard_core_query_execute(self->dbpriv->mgd, sql->str, TRUE);
-	g_string_free(sql, TRUE);
-
-	if(up > 0) {
-
-		midgard_replicator_export(MIDGARD_DBOBJECT(person));
-		return TRUE;
-	} 
-
-	return FALSE;	
-}
-
-/* Sets user's password. */
-gboolean midgard_user_password(MidgardUser *self, const gchar *login,
-				const gchar *password, guint hashtype)
-{
-	g_assert(self != NULL);
-
-	MidgardConnection *mgd = self->dbpriv->mgd;
-	MIDGARD_ERRNO_SET(mgd, MGD_ERR_OK);
-
-	if(self->priv->person == NULL) {
-		g_warning("Can not create user account. Person object is not assigned");
-		MIDGARD_ERRNO_SET(mgd, MGD_ERR_INTERNAL);
-		return FALSE;
-	}
-	
-	if(self->dbpriv->guid == NULL) {
-		g_warning("Can not change user account. Person object assigned with NULL guid");
-		MIDGARD_ERRNO_SET(mgd, MGD_ERR_INTERNAL);
-		return FALSE;
-	}
-
-	/* MidgardUser *user = MIDGARD_USER(mgd->priv->user); */
-	MidgardUser *user = self;
-
-	gint anon = 2;
-	
-	if(!midgard_user_is_admin(user))
-		anon--;
-
-	if(!midgard_user_is_user(user)) {	
-		anon--;
-	} else {
-		if(!g_str_equal(self->dbpriv->guid, 
-					user->dbpriv->guid))
-			anon--;
-	}
-
-	if(anon < 1) {
-		MIDGARD_ERRNO_SET(mgd, MGD_ERR_ACCESS_DENIED);
-		return FALSE;
-	}
-
-	/* Login can not be empty */
-	if(login == NULL || g_str_equal(login, "")) {
-		
-		 midgard_set_error(mgd,
-				 MGD_GENERIC_ERROR,
-				 MGD_ERR_INVALID_NAME,
-				 " Can not set empty login");
-		 return FALSE;
-	}
-
-	/* Password can not be empty, unless we use PAM */
-	if((password == NULL || g_str_equal(password, ""))
-			&& (hashtype != MIDGARD_USER_HASH_PAM)) {
-		
-		midgard_set_error(mgd,
-				MGD_GENERIC_ERROR,
-				MGD_ERR_INVALID_NAME,
-				" Can not set empty password");
-		return FALSE;
-	}
-
-	if(hashtype == MIDGARD_USER_HASH_LEGACY
-			|| hashtype == MIDGARD_USER_HASH_LEGACY_PLAIN) {
-		
-		return __set_legacy_password(self, login, password, hashtype);
-	}
-
-	if(password == NULL && hashtype == MIDGARD_USER_HASH_PAM)
-		password = "";
-
-	gchar *new_password;
-
-	switch(hashtype) {
-		
-		case MIDGARD_USER_HASH_MD5:
-			new_password = __string2md5hash(password);
-			break;
-			
-		case MIDGARD_USER_HASH_PLAIN:
-			new_password = g_strdup(password);
-			break;
-
-		case MIDGARD_USER_HASH_SHA1:
-			g_warning("SHA1 not yet supported");
-			return FALSE;
-			break;
-
-		case MIDGARD_USER_HASH_PAM:
-			new_password = g_strdup(password);
-			break;
-
-		default:
-			g_warning("Unsupported authentication type");
-			return FALSE;
-	}
-
-	MidgardQueryBuilder *builder;
-	GValue val = {0, };
-	gint rv;
-	GString *sql;
-
-	/* Let's check if person's record already exists */
-	builder = midgard_query_builder_new(mgd, "midgard_user");
-	g_value_init(&val, G_TYPE_STRING);
-	g_value_set_string(&val, login);
-	midgard_query_builder_add_constraint(builder, "login", "=", &val);
-	
-	guint count = midgard_query_builder_count(builder);
-
-	if(count == 0)
-		goto create_midgard_user;
-
-	/* Let's check if duplicate exists. */
-	builder = midgard_query_builder_new(mgd, "midgard_user");
-	g_value_init(&val, G_TYPE_STRING);
-	g_value_set_string(&val, login);
-	midgard_query_builder_add_constraint(builder, "login", "=", &val);
-
-	guint n_objects;
-	GObject **objects = 
-		midgard_query_builder_execute(builder, &n_objects);
-	g_object_unref(builder);
-
-	if(objects && (!g_str_equal(MIDGARD_DBOBJECT(objects[0])->dbpriv->guid, 
-				   self->dbpriv->guid))) {
-
-		MidgardUser *euser = MIDGARD_USER(objects[0]);
-		if(!g_str_equal(euser->dbpriv->guid, 
-					self->dbpriv->guid)) {
-			g_object_unref(objects[0]);
-			g_free(objects);
-			MIDGARD_ERRNO_SET(mgd, MGD_ERR_OBJECT_NAME_EXISTS);
-			return FALSE;
-		}
-
-		g_free(objects);
-	
-	} else {
-
-		sql = g_string_new("UPDATE midgard_user SET ");
-		g_string_append_printf(sql, 
-				"login = '%s', hashed = '%s', "
-				"hashtype = %d WHERE "
-				"guid = '%s'",
-				login, new_password, hashtype, 
-				self->dbpriv->guid);
-
-		rv =  midgard_core_query_execute(mgd, sql->str, FALSE);
-		g_string_free(sql, TRUE);
-		
-		if(rv == -1)
-			return FALSE;
-		
-		if(rv == -2) {
-			g_warning("Provider didn't return number of updated rows. midgard_user record might be not updated");
-			return TRUE;
-		}
-		
-		return TRUE;
-	}
-	
-	create_midgard_user:
-	sql = g_string_new("INSERT INTO midgard_user ");
-	g_string_append_printf(sql, 
-			"(guid, login, hashed, hashtype, "
-			"active, user_type) VALUES ('%s', '%s', '%s', "
-			"%d, %d)", 
-			self->dbpriv->guid, login, new_password, 
-			hashtype, MIDGARD_USER_TYPE_USER);
-	
-	rv = midgard_core_query_execute(mgd, sql->str, TRUE);
-	g_string_free(sql, TRUE);
-	
-	if(rv == -1)
-		return FALSE;
-	
-	if(rv == -2) {
-		g_warning("Provider didn't return number of updated rows. midgard_user record might be not updated");
-		return TRUE;
-	}
-	
-	return TRUE;
-}
-
 static void __set_from_sql(MidgardDBObject *object, 
 		GdaDataModel *model, gint row)
 {
@@ -1224,13 +965,12 @@ static void __set_from_sql(MidgardDBObject *object,
 	
 	/* 'at_col_name' acceptable since we do not use this frequently */
 	const GValue *value;
- 	guint i = 0;
-	gboolean active = FALSE;
+ 	guint i = 0;	
 
 	/* GUID */
 	value = midgard_data_model_get_value_at_col_name (model, "guid", row);
 	if (value)
-		self->dbpriv->guid = g_value_dup_string (value);
+		MIDGARD_DBOBJECT (self)->dbpriv->guid = g_value_dup_string (value);
 
 	/* LOGIN */
 	value = midgard_data_model_get_value_at_col_name (model, "login", row);
@@ -1246,7 +986,7 @@ static void __set_from_sql(MidgardDBObject *object,
 
 	/* PERSON GUID */
 	value = midgard_data_model_get_value_at_col_name (model, "person", row);
-	if (value && G_VALUE_HOLDS_STRING (value))
+	if (value && G_IS_VALUE (value) && G_VALUE_HOLDS_STRING (value))
 		self->priv->person_guid = g_value_dup_string (value);
 	else 
 		g_warning ("Invalid value type for user's person guid. Expected string, got %s", G_VALUE_TYPE_NAME (value));
@@ -1254,8 +994,7 @@ static void __set_from_sql(MidgardDBObject *object,
 	/* ACTIVE */
 	value = midgard_data_model_get_value_at_col_name (model, "active", row);
 	if (value) {
-		MIDGARD_GET_BOOLEAN_FROM_VALUE (active, value);
-		self->priv->active = active;
+		self->priv->active = g_value_get_boolean (value);
 	}
 
 	/* AUTH TYPE */
@@ -1286,6 +1025,9 @@ static void __set_from_sql(MidgardDBObject *object,
  *
  * Associates given #MidgardObject person with @self #MidgardUser.
  * Sets person property and updates user storage record.
+ *
+ * #MidgardUser @self takes ownership of the given #MidgardPerson reference, 
+ * which should be not unref any more.
  * 
  * See midgard_user_update() for returned error details.
  *
@@ -1314,24 +1056,28 @@ midgard_user_set_person (MidgardUser *self, MidgardObject *person)
  * midgard_user_get_person:
  * @self: #MidgardUser instance
  *
- * Returns: #MgdObject of "midgard_person" type, of %NULL if none associated.
+ * Returned object should not be unref.
+ *
+ * Returns: #MidgardObject of "midgard_person" type, of %NULL if none associated.
  */
-MgdObject *
+MidgardObject *
 midgard_user_get_person (MidgardUser *self)
 {
 	MidgardUserClass *klass = MIDGARD_USER_GET_CLASS (self);
 	return klass->get_person (self);	
 }
 
-MgdObject *
+MidgardObject *
 __midgard_user_get_person(MidgardUser *self)
 {
 	g_assert(self != NULL);
 
 	if (self->priv->person == NULL) {
 
+		gchar *person_guid = self->priv->person_guid;
+
 		/* Load person from storage if user holds its guid reference */
-		if (self->priv->person_guid) {
+		if (person_guid && *person_guid != '\0') {
 		
 			GValue gval = {0, };
 			g_value_init (&gval, G_TYPE_STRING);
@@ -1381,9 +1127,11 @@ midgard_user_quick_login (MidgardConnection *mgd, const gchar *login, const gcha
 	if (!user)
 		return NULL;
 
-	midgard_user_login (user);
+	if (midgard_user_log_in (user))
+		return user;
 
-	return user;
+	g_object_unref (user);
+	return NULL;
 }
 
 /* GOBJECT ROUTINES */
@@ -1394,8 +1142,8 @@ static void _midgard_user_finalize(GObject *object)
 	
 	MidgardUser *self = (MidgardUser *) object;
 	
-	if (self && (self->dbpriv && self->priv->is_logged))
-		(void) midgard_user_logout (self);
+	if (self && (MIDGARD_DBOBJECT (self)->dbpriv && self->priv->is_logged))
+		(void) midgard_user_log_out (self);
 
 	g_free (self->priv->auth_type);
 	self->priv->auth_type = NULL;
@@ -1475,7 +1223,7 @@ _midgard_user_get_property (GObject *object, guint property_id,
 	switch (property_id) {
 		
 		case MIDGARD_USER_GUID:
-			g_value_set_string(value, self->dbpriv->guid);
+			g_value_set_string(value, MIDGARD_DBOBJECT (self)->dbpriv->guid);
 			break;
 
 		case MIDGARD_USER_LOGIN:
@@ -1524,7 +1272,7 @@ __midgard_user_constructor (GType type,
 
 	MIDGARD_USER (object)->priv = g_new(MidgardUserPrivate, 1);
 	MIDGARD_USER (object)->priv->person = NULL;
-	MIDGARD_USER (object)->priv->person_guid = NULL;
+	MIDGARD_USER (object)->priv->person_guid = g_strdup ("");
 	MIDGARD_USER (object)->priv->user_type = MIDGARD_USER_TYPE_NONE;
 	MIDGARD_USER (object)->priv->fetched = FALSE;
 	MIDGARD_USER (object)->priv->login = NULL;
@@ -1540,6 +1288,9 @@ __midgard_user_constructor (GType type,
 static void
 __midgard_user_dispose (GObject *object)
 {
+	MidgardUser *self = MIDGARD_USER (object);	
+	if (self->priv->person && G_IS_OBJECT (self->priv->person))
+		g_object_unref (self->priv->person);
 	__parent_class->dispose (object);
 }
 
@@ -1577,8 +1328,8 @@ static void _midgard_user_class_init(
 
 	/* Virtual methods */
 	klass->get_person = __midgard_user_get_person;
-	klass->login = __midgard_user_login;
-	klass->logout = __midgard_user_logout;
+	klass->log_in = __midgard_user_login;
+	klass->log_out = __midgard_user_logout;
 	klass->get = __midgard_user_get;
 	klass->query = __midgard_user_query;
 	klass->create = __midgard_user_create;
@@ -1591,9 +1342,11 @@ static void _midgard_user_class_init(
 	MgdSchemaTypeAttr *type_attr = midgard_core_schema_type_attr_new();
 
 	GParamSpec *pspec;
+	gchar *property_name;
 
 	 /* GUID */
-	pspec = g_param_spec_string ("guid",
+	property_name = "guid";
+	pspec = g_param_spec_string (property_name,
 			"Guid which identifies user object.",
 			"",
 			"",
@@ -1603,14 +1356,16 @@ static void _midgard_user_class_init(
 			pspec);
 	prop_attr = midgard_core_schema_type_property_attr_new();
 	prop_attr->gtype = MGD_TYPE_GUID;
-	prop_attr->field = g_strdup("guid");
+	prop_attr->field = g_strdup(property_name);
 	prop_attr->table = g_strdup(MIDGARD_USER_TABLE);
-	prop_attr->tablefield = g_strjoin(".", MIDGARD_USER_TABLE, "guid", NULL);
+	prop_attr->tablefield = g_strjoin(".", MIDGARD_USER_TABLE, property_name, NULL);
 	g_hash_table_insert(type_attr->prophash,
-			g_strdup((gchar *)"guid"), prop_attr);
+			g_strdup((gchar *)property_name), prop_attr);
+	type_attr->_properties_list = g_slist_append (type_attr->_properties_list, property_name);
 
 	/* LOGIN */
-	pspec = g_param_spec_string ("login",
+	property_name = "login";
+	pspec = g_param_spec_string (property_name,
 			"midgard_user's login",
 			"",
 			"",
@@ -1620,14 +1375,15 @@ static void _midgard_user_class_init(
 			pspec);
 	prop_attr = midgard_core_schema_type_property_attr_new();
 	prop_attr->gtype = MGD_TYPE_STRING;
-	prop_attr->field = g_strdup("login");
+	prop_attr->field = g_strdup(property_name);
 	prop_attr->table = g_strdup(MIDGARD_USER_TABLE);
-	prop_attr->tablefield = g_strjoin(".", MIDGARD_USER_TABLE, "login", NULL);
+	prop_attr->tablefield = g_strjoin(".", MIDGARD_USER_TABLE, property_name, NULL);
 	prop_attr->dbindex = TRUE;
-	g_hash_table_insert(type_attr->prophash,
-			g_strdup((gchar *)"login"), prop_attr);
+	g_hash_table_insert(type_attr->prophash, g_strdup((gchar *)property_name), prop_attr);
+	type_attr->_properties_list = g_slist_append (type_attr->_properties_list, property_name);
 
 	 /* PASSWORD */
+	property_name = "password";
 	pspec = g_param_spec_string ("password",
 			"midgard_user's password",
 			"",
@@ -1638,14 +1394,15 @@ static void _midgard_user_class_init(
 			pspec);
 	prop_attr = midgard_core_schema_type_property_attr_new();
 	prop_attr->gtype = MGD_TYPE_STRING;
-	prop_attr->field = g_strdup("password");
+	prop_attr->field = g_strdup(property_name);
 	prop_attr->table = g_strdup(MIDGARD_USER_TABLE);
-	prop_attr->tablefield = g_strjoin(".", MIDGARD_USER_TABLE, "password", NULL);
-	g_hash_table_insert(type_attr->prophash,
-			g_strdup((gchar *)"password"), prop_attr);
-	
+	prop_attr->tablefield = g_strjoin(".", MIDGARD_USER_TABLE, property_name, NULL);
+	g_hash_table_insert(type_attr->prophash, g_strdup((gchar *)property_name), prop_attr);
+	type_attr->_properties_list = g_slist_append (type_attr->_properties_list, property_name);
+
        	/* PERSON */
-	pspec = g_param_spec_string ("person",
+	property_name = "person";
+	pspec = g_param_spec_string (property_name,
 			"Guid which identifies person object associated with user.",
 			"",
 			"",
@@ -1661,11 +1418,12 @@ static void _midgard_user_class_init(
 	prop_attr->is_link = TRUE;
 	prop_attr->link = g_strdup ("midgard_person");
 	prop_attr->link_target = g_strdup ("guid");
-	g_hash_table_insert(type_attr->prophash,
-			g_strdup((gchar *)"person"), prop_attr);
+	g_hash_table_insert(type_attr->prophash, g_strdup((gchar *)property_name), prop_attr);
+	type_attr->_properties_list = g_slist_append (type_attr->_properties_list, property_name);
 
 	/* ACTIVE */
-	pspec = g_param_spec_boolean ("active",
+	property_name = "active";
+	pspec = g_param_spec_boolean (property_name,
 			"midgard_user's active info",
 			"",
 			FALSE, G_PARAM_READWRITE);
@@ -1674,14 +1432,15 @@ static void _midgard_user_class_init(
 			pspec);
 	prop_attr = midgard_core_schema_type_property_attr_new();
 	prop_attr->gtype = MGD_TYPE_BOOLEAN;
-	prop_attr->field = g_strdup("active");
+	prop_attr->field = g_strdup(property_name);
 	prop_attr->table = g_strdup(MIDGARD_USER_TABLE);
-	prop_attr->tablefield = g_strjoin(".", MIDGARD_USER_TABLE, "active", NULL);
-	g_hash_table_insert(type_attr->prophash,
-			g_strdup((gchar *)"active"), prop_attr);
+	prop_attr->tablefield = g_strjoin(".", MIDGARD_USER_TABLE, property_name, NULL);
+	g_hash_table_insert(type_attr->prophash, g_strdup((gchar *)property_name), prop_attr);
+	type_attr->_properties_list = g_slist_append (type_attr->_properties_list, property_name);
 
 	 /* AUTH TYPE  */
-	pspec = g_param_spec_string ("authtype",
+	property_name = "authtype";
+	pspec = g_param_spec_string (property_name,
 			"midgard_user's authentication type",
 			"",
 			"",
@@ -1694,11 +1453,12 @@ static void _midgard_user_class_init(
 	prop_attr->field = g_strdup("auth_type");
 	prop_attr->table = g_strdup(MIDGARD_USER_TABLE);
 	prop_attr->tablefield = g_strjoin(".", MIDGARD_USER_TABLE, "auth_type", NULL);
-	g_hash_table_insert(type_attr->prophash,
-			g_strdup((gchar *)"authtype"), prop_attr);
+	g_hash_table_insert(type_attr->prophash, g_strdup((gchar *)property_name), prop_attr);
+	type_attr->_properties_list = g_slist_append (type_attr->_properties_list, property_name);
 
 	/* AUTH TYPE ID */
-	pspec = g_param_spec_uint ("authtypeid",
+	property_name = "authtypeid";
+	pspec = g_param_spec_uint (property_name,
 			"midgard_user's authentication type id",
 			"",
 			0, G_MAXUINT32, 0, G_PARAM_READABLE);
@@ -1711,11 +1471,12 @@ static void _midgard_user_class_init(
 	prop_attr->table = g_strdup(MIDGARD_USER_TABLE);
 	prop_attr->tablefield = g_strjoin(".", MIDGARD_USER_TABLE, "auth_type_id", NULL);
 	prop_attr->dbindex = TRUE;
-	g_hash_table_insert(type_attr->prophash,
-			g_strdup((gchar *)"authtypeid"), prop_attr);
+	g_hash_table_insert(type_attr->prophash, g_strdup((gchar *)property_name), prop_attr);
+	type_attr->_properties_list = g_slist_append (type_attr->_properties_list, property_name);
 
 	/* USER TYPE */
-	pspec = g_param_spec_uint ("usertype",
+	property_name = "usertype";
+	pspec = g_param_spec_uint (property_name,
 			"midgard_user's type",
 			"",
 			0, G_MAXUINT32, 0, G_PARAM_READWRITE);
@@ -1728,24 +1489,39 @@ static void _midgard_user_class_init(
 	prop_attr->table = g_strdup(MIDGARD_USER_TABLE);
 	prop_attr->tablefield = g_strjoin(".", MIDGARD_USER_TABLE, "user_type", NULL);
 	prop_attr->dbindex = TRUE;
-	g_hash_table_insert(type_attr->prophash,
-			g_strdup((gchar *)"usertype"), prop_attr);
+	g_hash_table_insert(type_attr->prophash, g_strdup((gchar *)property_name), prop_attr);
+	type_attr->_properties_list = g_slist_append (type_attr->_properties_list, property_name);
 
-	/* This must be replaced with GDA classes */
-	type_attr->sql_select_full = g_strdup("guid, login, password, auth_type as authtype, auth_type_id as authtypeid, active, person_guid as person, user_type AS usertype");
+	MIDGARD_DBOBJECT_CLASS (klass)->dbpriv = g_new(MidgardDBObjectPrivate, 1);
+	MIDGARD_DBOBJECT_CLASS (klass)->dbpriv->storage_data = type_attr;
+	MIDGARD_DBOBJECT_CLASS (klass)->dbpriv->storage_data->table = g_strdup(MIDGARD_USER_TABLE);
+	MIDGARD_DBOBJECT_CLASS (klass)->dbpriv->storage_data->tables = g_strdup(MIDGARD_USER_TABLE);
+	MIDGARD_DBOBJECT_CLASS (klass)->dbpriv->has_metadata = FALSE;
 
-	klass->dbpriv = g_new(MidgardDBObjectPrivate, 1);
-	klass->dbpriv->storage_data = type_attr;
-	klass->dbpriv->storage_data->table = g_strdup(MIDGARD_USER_TABLE);
-	klass->dbpriv->storage_data->tables = g_strdup(MIDGARD_USER_TABLE);
-	klass->dbpriv->has_metadata = FALSE;
+	MIDGARD_DBOBJECT_CLASS (klass)->dbpriv->__set_from_sql = __set_from_sql;
+	MIDGARD_DBOBJECT_CLASS (klass)->dbpriv->set_from_sql = NULL;
+	MIDGARD_DBOBJECT_CLASS (klass)->dbpriv->create_storage = midgard_core_query_create_class_storage;	
+	MIDGARD_DBOBJECT_CLASS (klass)->dbpriv->update_storage = midgard_core_query_update_class_storage;
+	MIDGARD_DBOBJECT_CLASS (klass)->dbpriv->storage_exists = _user_storage_exists; 
+	MIDGARD_DBOBJECT_CLASS (klass)->dbpriv->delete_storage = _user_storage_delete;
+	MIDGARD_DBOBJECT_CLASS (klass)->dbpriv->set_statement_insert = MIDGARD_DBOBJECT_CLASS (__parent_class)->dbpriv->set_statement_insert;
 
-	klass->dbpriv->__set_from_sql = __set_from_sql;
-	klass->dbpriv->set_from_sql = NULL;
-	klass->dbpriv->create_storage = midgard_core_query_create_class_storage;	
-	klass->dbpriv->update_storage = midgard_core_query_update_class_storage;
-	klass->dbpriv->storage_exists = _user_storage_exists; 
-	klass->dbpriv->delete_storage = _user_storage_delete;
+	type_attr->params = midgard_core_dbobject_class_list_properties (MIDGARD_DBOBJECT_CLASS (klass), &type_attr->num_properties);	
+	guint i;
+	const gchar *field_name;
+	GString *sql_full = g_string_new ("");
+
+	for (i = 0; i < type_attr->num_properties; i++) {
+		field_name = midgard_core_class_get_property_colname (MIDGARD_DBOBJECT_CLASS (klass), type_attr->params[i]->name);
+		if (i > 0)
+			g_string_append (sql_full, ", ");
+		g_string_append_printf (sql_full, "%s AS %s", field_name, type_attr->params[i]->name);
+	}
+	type_attr->sql_select_full = g_strdup (sql_full->str);
+	g_string_free (sql_full, TRUE);
+
+	 /* Initialize persistent statement */
+	MIDGARD_DBOBJECT_CLASS (klass)->dbpriv->set_statement_insert (MIDGARD_DBOBJECT_CLASS (klass));
 }
 
 static void _midgard_user_instance_init(
@@ -1770,9 +1546,7 @@ GType midgard_user_get_type(void)
 			0,              /* n_preallocs */
 			(GInstanceInitFunc) _midgard_user_instance_init/* instance_init */
 		};
-		type = g_type_register_static (MIDGARD_TYPE_DBOBJECT,
-				"midgard_user",
-				&info, 0);
+		type = g_type_register_static (MIDGARD_TYPE_DBOBJECT, "MidgardUser", &info, 0);
 	}
 	
 	return type;
