@@ -23,7 +23,6 @@
 #include "query_constraint.h"
 #include "group_constraint.h"
 #include "query_order.h"
-#include "midgard_object_class.h"
 #include "schema.h"
 #include "midgard_core_query_builder.h"
 #include "midgard_error.h"
@@ -39,6 +38,8 @@
 #include "midgard_timestamp.h"
 #include "midgard_core_metadata.h"
 #include "midgard_view.h"
+#include "midgard_reflector_object.h"
+#include "midgard_core_config.h"
 
 /* Internal prototypes , I am not sure if should be included in API */
 gchar *midgard_query_builder_get_object_select(MidgardQueryBuilder *builder, guint select_type);
@@ -249,20 +250,20 @@ gboolean midgard_query_builder_add_constraint(
         g_assert(op);
         g_assert(value);
      
-	MidgardQueryConstraint *constraint = midgard_query_constraint_new();
-	midgard_query_constraint_set_builder(constraint, builder);
-	midgard_query_constraint_set_class(constraint, MIDGARD_DBOBJECT_CLASS(g_type_class_peek(builder->priv->type)));
+	MidgardCoreQueryConstraint *constraint = midgard_core_query_constraint_new();
+	midgard_core_query_constraint_set_builder(constraint, builder);
+	midgard_core_query_constraint_set_class(constraint, MIDGARD_DBOBJECT_CLASS(g_type_class_peek(builder->priv->type)));
 	
-	if(!midgard_query_constraint_add_operator(constraint, op))
+	if(!midgard_core_query_constraint_add_operator(constraint, op))
 		return FALSE;
 		
-	if(!midgard_query_constraint_parse_property(&constraint, MIDGARD_DBOBJECT_CLASS(g_type_class_peek(builder->priv->type)), name))
+	if(!midgard_core_query_constraint_parse_property(&constraint, MIDGARD_DBOBJECT_CLASS(g_type_class_peek(builder->priv->type)), name))
 		return FALSE;
 
-	if(!midgard_query_constraint_add_value(constraint, value))
+	if(!midgard_core_query_constraint_add_value(constraint, value))
 		return FALSE;
 	
-	midgard_query_constraint_build_condition(constraint);
+	midgard_core_query_constraint_build_condition(constraint);
 
 	/* FIXME, table should be stored per every constraint, order, whatever */
 	midgard_core_qb_add_table(builder, constraint->priv->prop_left->table);
@@ -321,19 +322,19 @@ gboolean midgard_query_builder_add_constraint_with_property(
 
 	MidgardDBObjectClass *klass = MIDGARD_DBOBJECT_CLASS(g_type_class_peek(builder->priv->type));
 
-	MidgardQueryConstraint *constraint = midgard_query_constraint_new();
-	midgard_query_constraint_set_builder(constraint, builder);
-	midgard_query_constraint_set_class(constraint, klass);
+	MidgardCoreQueryConstraint *constraint = midgard_core_query_constraint_new();
+	midgard_core_query_constraint_set_builder(constraint, builder);
+	midgard_core_query_constraint_set_class(constraint, klass);
 
-	if(!midgard_query_constraint_parse_property(&constraint, klass, property_a))
+	if(!midgard_core_query_constraint_parse_property(&constraint, klass, property_a))
 		return FALSE;
 	constraint->priv->current = constraint->priv->prop_right;
-	if(!midgard_query_constraint_parse_property(&constraint, klass, property_b))
+	if(!midgard_core_query_constraint_parse_property(&constraint, klass, property_b))
 		return FALSE;
 
 	constraint->priv->condition_operator = g_strdup(op);
 
-	midgard_query_constraint_build_condition(constraint);
+	midgard_core_query_constraint_build_condition(constraint);
 
 	midgard_core_qb_add_table(builder, constraint->priv->prop_left->table);
 	midgard_core_qb_add_table(builder, constraint->priv->prop_right->table);
@@ -555,7 +556,7 @@ static GList *midgard_query_builder_execute_or_count(MidgardQueryBuilder *builde
 
 			GValue gval = {0, };
 			g_value_init(&gval, G_TYPE_STRING);
-			g_value_set_string(&gval, user->dbpriv->guid);
+			g_value_set_string(&gval, MIDGARD_DBOBJECT (user)->dbpriv->guid);
 
 			midgard_query_builder_add_constraint(builder, "guid", "=", &gval);
 		}
@@ -618,7 +619,7 @@ GObject **midgard_query_builder_execute(
 	}
 
 	*n_objects = holder->elements;
-	MgdObject **objects = g_new(MgdObject *, holder->elements+1);	
+	MidgardObject **objects = g_new(MidgardObject *, holder->elements+1);	
 
 	for( ; list; list = list->next){
 		objects[i] = list->data;
@@ -692,7 +693,7 @@ gchar *midgard_query_builder_get_object_select(
 		return g_string_free(select, FALSE);
 	}
 	
-        /* guid hardcoded ( inherited from MgdObjectClass )
+        /* guid hardcoded ( inherited from MidgardObjectClass )
          * metadata properties hardcoded ( defined in midgard_metadata )
          */ 
 	/* FIXME, move this to particular class implementation, so we can reuse MidgardDBObject derived class data only */
@@ -763,13 +764,8 @@ static void __mqb_set_metadata(MidgardMetadata *mdata, GdaDataModel *model, gint
 	GValue mt = {0, };
 
 	/* creator */
-	gvalue = midgard_data_model_get_value_at(model, ++col, i);
-	g_free(mdata->priv->creator);
-	mdata->priv->creator = NULL;
-	if(G_VALUE_HOLDS_STRING (gvalue)) {
-		mdata->priv->creator =
-			g_value_dup_string(gvalue);
-	}
+	gvalue = midgard_data_model_get_value_at(model, ++col, i);	
+	midgard_core_metadata_set_creator (mdata, gvalue);
 
 	/* created */	
 	gvalue = midgard_data_model_get_value_at(model, ++col, i);
@@ -777,12 +773,8 @@ static void __mqb_set_metadata(MidgardMetadata *mdata, GdaDataModel *model, gint
 
 	/* revisor */
 	gvalue = midgard_data_model_get_value_at(model, ++col, i);
-	g_free(mdata->priv->revisor);
-	mdata->priv->revisor = NULL;
-	if(G_VALUE_HOLDS_STRING (gvalue)) {
-		mdata->priv->revisor = 
-			g_value_dup_string(gvalue);
-	}
+	if(G_VALUE_HOLDS_STRING (gvalue)) 
+		midgard_core_metadata_set_revisor (mdata, gvalue);
 		
 	/* revised */
 	gvalue = midgard_data_model_get_value_at(model, ++col, i);
@@ -795,12 +787,8 @@ static void __mqb_set_metadata(MidgardMetadata *mdata, GdaDataModel *model, gint
 		
 	/* locker */
 	gvalue = midgard_data_model_get_value_at(model, ++col, i);
-	g_free(mdata->priv->locker);
-	mdata->priv->locker = NULL;
-	if(G_VALUE_HOLDS_STRING (gvalue)) {
-		mdata->priv->locker = 
-			g_value_dup_string(gvalue);
-	}
+	if(G_VALUE_HOLDS_STRING (gvalue)) 
+		midgard_core_metadata_set_locker (mdata, gvalue);
 		
 	/* locked */
 	gvalue = midgard_data_model_get_value_at(model, ++col, i);
@@ -808,12 +796,9 @@ static void __mqb_set_metadata(MidgardMetadata *mdata, GdaDataModel *model, gint
 
 	/* approver */
 	gvalue = midgard_data_model_get_value_at(model, ++col, i);
-	g_free(mdata->priv->approver);
-	mdata->priv->approver = NULL;
-	if(G_VALUE_HOLDS_STRING (gvalue)) {
-		mdata->priv->approver = 
-			g_value_dup_string(gvalue);
-	}
+	if(G_VALUE_HOLDS_STRING (gvalue)) 
+		midgard_core_metadata_set_approver (mdata, gvalue);
+
 
 	/* approved */
 	gvalue = midgard_data_model_get_value_at(model, ++col, i);
@@ -821,35 +806,21 @@ static void __mqb_set_metadata(MidgardMetadata *mdata, GdaDataModel *model, gint
 
 	/* authors */
 	gvalue = midgard_data_model_get_value_at(model, ++col, i);
-	g_free(mdata->priv->authors);
-	mdata->priv->authors = NULL;
-	if(G_VALUE_HOLDS_STRING (gvalue)) {
-		mdata->priv->authors = 
-			g_value_dup_string(gvalue);
-	}
+	if(G_VALUE_HOLDS_STRING (gvalue)) 
+		midgard_core_metadata_set_authors (mdata, gvalue);
 		
 	/* owner */
 	gvalue = midgard_data_model_get_value_at(model, ++col, i);
-	g_free(mdata->priv->owner);
-	mdata->priv->owner = NULL;
-	if(G_VALUE_HOLDS_STRING (gvalue)) {
-		mdata->priv->owner = 
-			g_value_dup_string(gvalue);
-	}
+	if(G_VALUE_HOLDS_STRING (gvalue)) 
+		midgard_core_metadata_set_owner (mdata, gvalue);
 
 	/* schedule_start */
 	gvalue = midgard_data_model_get_value_at(model, ++col, i);
-	g_value_init(&mt, MGD_TYPE_TIMESTAMP); 
-	g_value_transform(gvalue, &mt); 
-	mdata->priv->schedule_start = g_value_dup_boxed(&mt);
-	g_value_unset(&mt);
+	_SET_METADATA_TIMESTAMP (schedule_start);
 
 	/* schedule_end */
 	gvalue = midgard_data_model_get_value_at(model, ++col, i);
-	g_value_init(&mt, MGD_TYPE_TIMESTAMP); 
-	g_value_transform(gvalue, &mt); 
-	mdata->priv->schedule_end = g_value_dup_boxed(&mt);
-	g_value_unset(&mt);
+	_SET_METADATA_TIMESTAMP (schedule_end);
 
 	/* hidden */
 	gvalue = midgard_data_model_get_value_at(model, ++col, i);
@@ -876,10 +847,7 @@ static void __mqb_set_metadata(MidgardMetadata *mdata, GdaDataModel *model, gint
 
 	/* published */
 	gvalue = midgard_data_model_get_value_at(model, ++col, i);
-	g_value_init(&mt, MGD_TYPE_TIMESTAMP); 
-	g_value_transform(gvalue, &mt); 
-	mdata->priv->published = g_value_dup_boxed(&mt);
-	g_value_unset(&mt);
+	_SET_METADATA_TIMESTAMP (published);
 
 	/* exported */
 	gvalue = midgard_data_model_get_value_at(model, ++col, i);
@@ -922,12 +890,60 @@ static void __mqb_set_metadata(MidgardMetadata *mdata, GdaDataModel *model, gint
 	return;
 }
 
+
+/* based on gda_default_unescape_string */
+/* https://bugzilla.gnome.org/show_bug.cgi?id=617565 */
+gchar *
+__default_unescape_string (const gchar *string)
+{
+	glong total;
+	gchar *ptr;
+	gchar *retval;
+	glong offset = 0;
+	
+	if (!string) 
+		return NULL;
+
+	total = strlen (string);
+	retval = g_memdup (string, total+1);
+	ptr = (gchar *) retval;
+	while (offset < total) {
+
+		/* we accept the "''" as a synonym of "\'" */
+		if (*ptr == '\'') {
+			if (*(ptr+1) == '\'') {
+				g_memmove (ptr+1, ptr+2, total - offset);
+				offset += 2;
+			}
+		}
+		if (*ptr == '\\') {
+			if (*(ptr+1) == '\\') {
+				g_memmove (ptr+1, ptr+2, total - offset);
+				offset += 2;
+			}
+			else {
+				if (*(ptr+1) == '\'') {
+					*ptr = '\'';
+					g_memmove (ptr+1, ptr+2, total - offset);
+					offset += 2;
+				}				
+			}
+		}
+		else
+			offset ++;
+
+		ptr++;
+	}
+
+	return retval;	
+}
+
+
 GList *
-midgard_core_qb_set_object_from_query (MidgardQueryBuilder *builder, guint select_type, MgdObject *nobject)
+midgard_core_qb_set_object_from_query (MidgardQueryBuilder *builder, guint select_type, MidgardObject **nobject)
 {
         g_assert(builder != NULL);
-
-        MgdObject *object;
+ 
         guint ret_rows, ret_fields;
 	MidgardConnection *mgd = builder->priv->mgd;
       
@@ -961,6 +977,7 @@ midgard_core_qb_set_object_from_query (MidgardQueryBuilder *builder, guint selec
 	if(!model) 
 		return NULL;	
 
+	MidgardObject *object = NULL;
 	gint rows, columns;
 	const GValue *gvalue = NULL;
 
@@ -1005,10 +1022,10 @@ midgard_core_qb_set_object_from_query (MidgardQueryBuilder *builder, guint selec
 	/* Get every row */
 	for (rows = 0; rows < ret_rows; rows++) {
 	
-		if(nobject)
-			object = nobject;
-		else
+		if(!nobject)
 			object = g_object_new(builder->priv->type, NULL);
+		else 
+			object = *nobject;
 
 		MIDGARD_DBOBJECT(object)->dbpriv->mgd = builder->priv->mgd;
 				
@@ -1024,7 +1041,10 @@ midgard_core_qb_set_object_from_query (MidgardQueryBuilder *builder, guint selec
 
 			if (dbklass->dbpriv->has_metadata) {
 
-				GObjectClass *metadata_class = midgard_object_class_get_metadata_class (MIDGARD_OBJECT_CLASS (dbklass));
+				/* FIXME, provide metadata class pointer member for MidgardDBObjectClass */
+				const gchar *metadata_classname = 
+					midgard_reflector_object_get_metadata_class (G_OBJECT_CLASS_NAME (dbklass));
+				GObjectClass *metadata_class = g_type_class_peek (g_type_from_name (metadata_classname));
 				GParamSpec **pspec = g_object_class_list_properties (metadata_class, &n_md_props);
 				g_free (pspec);
 			}
@@ -1049,20 +1069,29 @@ midgard_core_qb_set_object_from_query (MidgardQueryBuilder *builder, guint selec
 						continue;
 					}
 
-					if (G_VALUE_TYPE (gvalue) == GDA_TYPE_BINARY 
+					if (G_VALUE_TYPE (gvalue) == GDA_TYPE_BINARY
 							&& G_TYPE_FUNDAMENTAL (pspec->value_type) == G_TYPE_STRING) {
 
 						gchar *stringified = midgard_core_query_binary_stringify ((GValue *)gvalue);
 						g_object_set (G_OBJECT (object), coltitle, stringified, NULL);
-						g_free (stringified);	
-	
+				   		g_free (stringified);
+					
 					} else if (pspec->value_type != G_VALUE_TYPE (gvalue)) {
 
 						GValue _convert = {0, };
 						g_value_init (&_convert, pspec->value_type);	
-		
+				
 						if (g_value_transform (gvalue, &_convert)) {
 
+							/* FIXME, remove workaround once it's fixed in GDA */
+							/* https://bugzilla.gnome.org/show_bug.cgi?id=617550 */
+							guint dbtype = mgd->priv->config->priv->dbtype;
+							if (dbtype == MIDGARD_DB_TYPE_MYSQL && G_VALUE_TYPE (gvalue) == GDA_TYPE_BLOB) {
+								gchar *tmp_str = __default_unescape_string (g_value_get_string (&_convert));
+								if (tmp_str)
+									g_value_take_string (&_convert, tmp_str);
+							}
+
 							g_object_set_property (G_OBJECT (object), coltitle, &_convert);
 					
 						} else {
@@ -1114,16 +1143,11 @@ midgard_core_qb_set_object_from_query (MidgardQueryBuilder *builder, guint selec
 		}
 		
 		/* Set metadata */
-	     	if (dbklass->dbpriv->has_metadata) {
-	
+		MidgardMetadata *metadata = MGD_DBOBJECT_METADATA (object);
+	     	if (metadata) {
 			GParamSpec *pspec = g_object_class_find_property(G_OBJECT_GET_CLASS(object), "metadata");
-
-			if(pspec){
-	
-				MidgardMetadata *mdata = object->metadata;	
-				if(mdata)
-					__mqb_set_metadata(mdata, model, rows);
-			}
+			if (pspec)
+				__mqb_set_metadata(metadata, model, rows);
 		}	
 
 		list = g_list_append(list, G_OBJECT(object));                		
@@ -1241,9 +1265,7 @@ GType midgard_query_builder_get_type(void)
 			0,              /* n_preallocs */
 			(GInstanceInitFunc) _midgard_query_builder_instance_init /* instance_init */
 		};
-		type = g_type_register_static (G_TYPE_OBJECT,
-				"midgard_query_builder",
-				&info, 0);
+		type = g_type_register_static (G_TYPE_OBJECT, "MidgardQueryBuilder", &info, 0);
 	}
 	return type;
 }
