@@ -23,24 +23,21 @@
 #include "midgard_core_query_builder.h"
 #include "midgard_core_object.h"
 #include "midgard_core_object_class.h"
-#include "midgard_object_class.h"
-#include "midgard_tree.h"
 #include <libgda/libgda.h>
-#include "midgard_core_query.h"
 
-static void add_sql(MidgardQueryConstraint *constraint, GString *sql) {
+static void add_sql(MidgardCoreQueryConstraint *constraint, GString *sql) {
         g_assert(constraint);
         g_assert(sql);
         g_string_append(sql, "1=1");
 }
 
-G_DEFINE_TYPE(MidgardQueryConstraint, midgard_query_constraint, G_TYPE_OBJECT)
+G_DEFINE_TYPE(MidgardCoreQueryConstraint, midgard_core_query_constraint, G_TYPE_OBJECT)
 
-void midgard_query_constraint_add_sql(
-                MidgardQueryConstraint *constraint, GString *sql) {
+void midgard_core_query_constraint_add_sql(
+                MidgardCoreQueryConstraint *constraint, GString *sql) {
         g_assert(constraint);
         g_assert(sql);
-        MIDGARD_QUERY_CONSTRAINT_GET_CLASS(constraint)->add_sql(constraint, sql);
+        MIDGARD_CORE_QUERY_CONSTRAINT_GET_CLASS(constraint)->add_sql(constraint, sql);
 }
 
 static const char *valid_operator[] = {
@@ -48,9 +45,9 @@ static const char *valid_operator[] = {
 	NULL
 };
 
-gboolean midgard_query_constraint_add_operator(MidgardQueryConstraint *constraint, const gchar *op)
+gboolean midgard_core_query_constraint_add_operator(MidgardCoreQueryConstraint *constraint, const gchar *op)
 {
-	if(!midgard_query_constraint_operator_is_valid(op))
+	if(!midgard_core_query_constraint_operator_is_valid(op))
 		return FALSE;
 	
 	constraint->priv->condition_operator = g_strdup(op);
@@ -58,7 +55,7 @@ gboolean midgard_query_constraint_add_operator(MidgardQueryConstraint *constrain
 	return TRUE;
 }
 
-gboolean midgard_query_constraint_operator_is_valid(const gchar *op)
+gboolean midgard_core_query_constraint_operator_is_valid(const gchar *op)
 {
 	guint i;
 	for (i = 0; valid_operator[i] != NULL; i++) {
@@ -78,8 +75,8 @@ gboolean midgard_query_constraint_operator_is_valid(const gchar *op)
 	return TRUE;
 }
 
-gboolean midgard_query_constraint_add_value(
-		MidgardQueryConstraint *constraint, const GValue *value)
+gboolean midgard_core_query_constraint_add_value(
+		MidgardCoreQueryConstraint *constraint, const GValue *value)
 {
 	g_assert(constraint != NULL);
 	g_assert(value != NULL);
@@ -189,25 +186,21 @@ gboolean midgard_query_constraint_add_value(
 }
 
 static void __condition_append_value(GString *str, 
-		MidgardQueryConstraint *constraint, GValue *value)
+		MidgardCoreQueryConstraint *constraint, GValue *value)
 {
 	gchar *escaped;
-	MidgardConnection *mgd = NULL;
-	if (constraint->priv->builder)
-		mgd = constraint->priv->builder->priv->mgd;
-
 	if(value == NULL)
 		value = constraint->priv->value;
 
 	switch(G_VALUE_TYPE(value)) {
 
 		case G_TYPE_STRING:
-			if (mgd)
-				escaped = midgard_core_query_escape_string (mgd, g_value_get_string (value));
-			else 
-				escaped = gda_default_escape_string (g_value_get_string (value));
-			g_string_append_printf(str, "'%s'", escaped);
-			g_free (escaped);
+			escaped =
+				gda_connection_value_to_sql_string(
+						constraint->priv->builder->priv->mgd->priv->connection,
+						value);
+			g_string_append_printf(str, "%s", escaped);
+			g_free(escaped);
 			break;
 
 		case G_TYPE_UINT:
@@ -251,7 +244,7 @@ static void __condition_append_value(GString *str,
 	}
 }
 
-static void __transform_value(MidgardQueryConstraint *self)
+static void __transform_value(MidgardCoreQueryConstraint *self)
 {
 	GValue *value = self->priv->value;
 	GValue *target = g_new0(GValue, 1);
@@ -302,8 +295,8 @@ static void __transform_value(MidgardQueryConstraint *self)
 	return;
 }
 
-gboolean midgard_query_constraint_build_condition(
-		MidgardQueryConstraint *constraint)
+gboolean midgard_core_query_constraint_build_condition(
+		MidgardCoreQueryConstraint *constraint)
 {
 	g_assert(constraint != NULL);
 	
@@ -447,7 +440,7 @@ gboolean midgard_query_constraint_build_condition(
 }
 
 void            
-midgard_query_constraint_set_builder (MidgardQueryConstraint *self, MidgardQueryBuilder *builder)
+midgard_core_query_constraint_set_builder (MidgardCoreQueryConstraint *self, MidgardQueryBuilder *builder)
 {
 	g_assert(self != NULL);
 	g_assert(builder != NULL);
@@ -456,7 +449,7 @@ midgard_query_constraint_set_builder (MidgardQueryConstraint *self, MidgardQuery
 }
 
 void            
-midgard_query_constraint_set_class (MidgardQueryConstraint *self, MidgardDBObjectClass *klass)
+midgard_core_query_constraint_set_class (MidgardCoreQueryConstraint *self, MidgardDBObjectClass *klass)
 {
 	g_assert(self != NULL);
 	g_assert(klass != NULL);
@@ -466,12 +459,12 @@ midgard_query_constraint_set_class (MidgardQueryConstraint *self, MidgardDBObjec
 
 /* GOBJECT ROUTINES */
 
-MidgardQueryConstraintPrivate *
-midgard_query_constraint_private_new (void)
+MidgardCoreQueryConstraintPrivate *
+midgard_core_query_constraint_private_new (void)
 {
-	MidgardQueryConstraintPrivate *priv = g_new (MidgardQueryConstraintPrivate, 1);
+	MidgardCoreQueryConstraintPrivate *priv = g_new (MidgardCoreQueryConstraintPrivate, 1);
 	
-	/* Initialize empty property attr. We need only pointers. */
+	/* We need empty structures. We will reuse available pointers */
 	priv->prop_left = midgard_core_schema_type_property_attr_new_empty();
 	priv->prop_right = midgard_core_schema_type_property_attr_new_empty();
 
@@ -493,14 +486,14 @@ midgard_query_constraint_private_new (void)
 	return priv;
 }
 
-void midgard_query_constraint_private_free(MidgardQueryConstraintPrivate *mqcp)
+void midgard_core_query_constraint_private_free(MidgardCoreQueryConstraintPrivate *mqcp)
 {
 	g_assert(mqcp != NULL);
 
-	/* Just free structures, they are initialized empty. */
+	/* Just free, they were initialized empty */
 	g_free (mqcp->prop_left);
 	g_free (mqcp->prop_right);
-	
+
 	if(mqcp->condition_operator != NULL)
 		g_free(mqcp->condition_operator);
 
@@ -519,42 +512,42 @@ void midgard_query_constraint_private_free(MidgardQueryConstraintPrivate *mqcp)
 	GSList *list;
 	
 	for(list = mqcp->joins; list != NULL; list = list->next) {
-		midgard_query_constraint_private_free(
-				(MidgardQueryConstraintPrivate*)list->data);
+		midgard_core_query_constraint_private_free(
+				(MidgardCoreQueryConstraintPrivate*)list->data);
 	}
 	g_slist_free(list);
 
 	g_free(mqcp);
 }
 
-static void midgard_query_constraint_init(MidgardQueryConstraint *self) 
+static void midgard_core_query_constraint_init(MidgardCoreQueryConstraint *self) 
 {
 	g_assert(self != NULL);
 
-	self->priv = midgard_query_constraint_private_new();
+	self->priv = midgard_core_query_constraint_private_new();
 }
 
-static void midgard_query_constraint_finalize(GObject *object)
+static void midgard_core_query_constraint_finalize(GObject *object)
 {
 	g_assert(object != NULL);
 
-	MidgardQueryConstraint *self = MIDGARD_QUERY_CONSTRAINT(object);
-	midgard_query_constraint_private_free(self->priv);		
+	MidgardCoreQueryConstraint *self = MIDGARD_CORE_QUERY_CONSTRAINT(object);
+	midgard_core_query_constraint_private_free(self->priv);		
 }
 
-static void midgard_query_constraint_class_init(
-                MidgardQueryConstraintClass *klass) 
+static void midgard_core_query_constraint_class_init(
+                MidgardCoreQueryConstraintClass *klass) 
 {
         g_assert(klass != NULL);
 	
-	klass->parent.finalize = midgard_query_constraint_finalize;
+	klass->parent.finalize = midgard_core_query_constraint_finalize;
 	klass->add_sql = add_sql;
 }
 
-MidgardQueryConstraint *midgard_query_constraint_new(void)
+MidgardCoreQueryConstraint *midgard_core_query_constraint_new(void)
 {
-	MidgardQueryConstraint *self = 
-		g_object_new(MIDGARD_TYPE_QUERY_CONSTRAINT, NULL);
+	MidgardCoreQueryConstraint *self = 
+		g_object_new(MIDGARD_CORE_TYPE_QUERY_CONSTRAINT, NULL);
 
 	return self;
 }
