@@ -46,10 +46,10 @@ struct _MgdSchemaPropertyAttr{
 	gboolean is_linked;
 	gboolean is_primary;
 	gboolean is_reversed;
+	gboolean is_private;
+	gboolean is_unique;
 	gchar *description;
 	GHashTable *user_values;
-	gboolean legacy_multilang;
-	gchar *legacy_field;
 };
 
 struct _MgdSchemaTypeAttr {
@@ -80,18 +80,20 @@ struct _MgdSchemaTypeAttr {
 	GSList *constraints;
 	gboolean is_view;
 	gchar *sql_create_view;
-	gchar *metadata_class;
-	GHashTable *user_values;
-	gboolean legacy_multilang;	
+	gchar *metadata_class_name;
+	MidgardMetadataClass *metadata_class_ptr;
+	GHashTable *user_values;	
 };
 
+#define MGD_TYPE_ATTR_METADATA_CLASS(__typeattr) __typeattr->metadata_class_name
+
 /* MgdSchema storage utilities */
 MgdSchemaTypeAttr 	*midgard_schema_lookup_type		(MidgardSchema *schema, gchar *type);
 GType			midgard_type_register			(MgdSchemaTypeAttr *data, GType parent_type);
 
-MgdSchemaTypeAttr	*midgard_core_schema_type_attr_new		(void);
-void 			midgard_core_schema_type_attr_free		(MgdSchemaTypeAttr *prop);
-void			midgard_core_schema_type_attr_extend 		(MgdSchemaTypeAttr *src, MgdSchemaTypeAttr *dst);
+MgdSchemaTypeAttr	*midgard_core_schema_type_attr_new	(void);
+void 			midgard_core_schema_type_attr_free	(MgdSchemaTypeAttr *prop);
+void                    midgard_core_schema_type_attr_extend	(MgdSchemaTypeAttr *src, MgdSchemaTypeAttr *dst);
 
 void			midgard_core_schema_type_set_table	 	(MgdSchemaTypeAttr *prop, const gchar *table);
 void 			midgard_core_schema_type_add_table 		(MgdSchemaTypeAttr *type, const gchar *table);
@@ -108,8 +110,6 @@ void 			midgard_core_schema_type_property_set_tablefield	(MgdSchemaPropertyAttr
 void			midgard_core_schema_type_property_set_table 		(MgdSchemaPropertyAttr *prop, const gchar *table);
 MgdSchemaPropertyAttr	*midgard_core_schema_type_property_lookup		(MgdSchemaTypeAttr *type, const gchar *name);
 
-
-
 GType 			midgard_core_schema_gtype_from_string		(const gchar *type);
 
 #define MGD_SCHEMA_XML_NAMESPACE "http:/""/www.midgard-project.org/repligard/1.4"
@@ -126,4 +126,13 @@ GType 			midgard_core_schema_gtype_from_string		(const gchar *type);
 #define TYPE_RW_USERVALUES	"user_values"
 #define TYPE_RW_PROPERTY	"property"
 
+#define PROP_RW_PRIVATE		"private"
+#define PROP_RW_FIELD		"field"
+#define PROP_RW_INDEX		"index"
+#define PROP_RW_UPFIELD		"upfield"
+#define PROP_RW_PARENTFIELD	"parentfield"
+#define PROP_RW_PRIMARY		"primaryfield"
+#define PROP_RW_REVERSE		"reverse"
+#define PROP_RW_LINK		"link"
+
 #endif /* _PRIVATE_SCHEMA_H */
